//
//  Controller.m
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//


#import "Controller.h"
#import "Game.h"

//#include <QuickTime/QuickTime.h>

@implementation Controller

- (id) init
{
    self = [super init];


    game = [[[Game alloc] init] retain];

    // load up the highscore table
    hiScores = [[NSUserDefaults standardUserDefaults] arrayForKey:@"hiScores"];
    if ((!hiScores)||([hiScores count] < 8))
    {
        hiScores = [self makeBlankHiScoresWith:hiScores];
        [[NSUserDefaults standardUserDefaults] setObject:hiScores forKey:@"hiScores"];
    }
    [hiScores retain];

    gameNames = [hiScores objectAtIndex:0];
    gameScores = [hiScores objectAtIndex:1];
    gameLevels = [hiScores objectAtIndex:2];

    return self;
}

- (void) dealloc
{
    if(game) [game release];

    [self dealloc];
}

- (void) awakeFromNib
{
    [gameWindow setFrameAutosaveName:@"gameWindow"];

    [gameView setController:self];
    [gameView setGame:game];
    [gameView setNeedsDisplay:YES];

    // fudge for on screen high score
    [self checkHighScores];

    // audio settings
    [game setMusicOn:[[NSUserDefaults standardUserDefaults] boolForKey:@"MusicOn"]];

    [game setSoundOn:[[NSUserDefaults standardUserDefaults] boolForKey:@"SoundOn"]];
    
    //musicPlayer = nil;
    /* retrieve our movie file from our bundle resource */
    NSString *filePath = [[NSBundle mainBundle] pathForResource:@"gp_menu_drums" ofType:@"mp3"];
    NSURL *movieUrl = [NSURL fileURLWithPath:filePath];

    /* create an NSMovie object from our QuickTime movie */
    NSMovie *song2 = [[NSMovie alloc] initWithURL:movieUrl byReference:YES];

    [musicPlayer setMovie:song2];

    [musicPlayer gotoBeginning:self];

    if([game isMusicOn])
    {
        [musicPlayer start:self];
    }
    else
    {
        [musicPlayer stop:self];
    }
}

- (void)musicOnOff:(bool)on
{
    if(on)
    {
        [musicPlayer start:self];
    }
    else
    {
        [musicPlayer stop:self];
    }
}

- (IBAction)fullScreenOnOff:(id)sender
{
    int windowLevel;
    NSRect screenRect;

    if(boolFullScreen)
    {
        boolFullScreen = NO;
        [mainWindow orderOut:self];

        CGDisplaySwitchToMode (kCGDirectMainDisplay, originalMode);
        //CFRelease (originalMode);
        CGDisplayRelease (kCGDirectMainDisplay);
        CGDisplayShowCursor (kCGDirectMainDisplay);

        // Get the screen rect of our main display
        screenRect = [gameWindow frame];


        NSRect gameRect;
        // set the rectangle for the card
        gameRect.origin.x =  0;
        gameRect.origin.y = 0;
        gameRect.size.width = 800;
        gameRect.size.height = 600;

        [gameWindow setContentView:gameView];

        [gameView setFrame:gameRect];





        /*
         // Release the display(s)
         if (CGDisplayRelease( kCGDirectMainDisplay ) != kCGErrorSuccess) {
             NSLog( @"Couldn't release the display(s)!" );
             // Note: if you display an error dialog here, make sure you set
             // its window level to the same one as the shield window level,
             // or the user won't see anything.
         }
         */


    }
    else
    {
        boolFullScreen = YES;

        originalMode = CGDisplayCurrentMode (kCGDirectMainDisplay);

        //CGDisplayHideCursor (kCGDirectMainDisplay);



        CFDictionaryRef newMode =  CGDisplayBestModeForParameters(kCGDirectMainDisplay, 24, 800, 600, 0);
        NSAssert(newMode, @"Couldn't find display mode");



        //fade out
        CGDisplayReservationInterval seconds = 2.0;
        CGDisplayFadeReservationToken newToken;
        CGAcquireDisplayFadeReservation(seconds, &newToken); // reserve display hardware time

        CGDisplayFade(newToken,
                      0.3,                          // 0.3 seconds
                      kCGDisplayBlendNormal,        // Starting state
                      kCGDisplayBlendSolidColor,    // Ending state
                      0.0, 0.0, 0.0,                // black
                      true);                        // wait for completion

        CGDisplayCapture(kCGDirectMainDisplay);             //capture main display

        //Switch to selected resolution.
        CGDisplayErr err = CGDisplaySwitchToMode(kCGDirectMainDisplay, newMode);
        NSAssert(err == CGDisplayNoErr, @"Error switching resolution.");

        CGDisplayFade(newToken,
                      0.5,                          // 0.5 seconds
                      kCGDisplayBlendSolidColor,    // Starting state
                      kCGDisplayBlendNormal,        // Ending state
                      0.0, 0.0, 0.0,                // black
                      false);                       // Don't wait for completion

        CGReleaseDisplayFadeReservation(newToken);


        /*
         // Capture the main display
         if (CGDisplayCapture( kCGDirectMainDisplay ) != kCGErrorSuccess)
         {
             NSLog( @"Couldn't capture the main display!" );
             // Note: you'll probably want to display a proper error dialog here
         }
         */

        NSLog(@"Full screen code entered");

        // Get the shielding window level
        windowLevel = CGShieldingWindowLevel();

        // Get the screen rect of our main display
        screenRect = [[NSScreen mainScreen] frame];

        //[gameWindow setLevel:windowLevel];

        NSRect gameRect;
        // set the rectangle for the card
        gameRect.origin.x =  200;
        gameRect.origin.y = 0;
        gameRect.size.width = 800;
        gameRect.size.height = 600;



        // Put up a new window
        mainWindow = [[NSWindow alloc] initWithContentRect:screenRect
                                                 styleMask:NSTitledWindowMask //NSBorderlessWindowMask
                                                   backing:NSBackingStoreBuffered
                                                     defer:NO];


        [mainWindow setTitle:@"myWindow"];
        [mainWindow setReleasedWhenClosed:YES];
        [mainWindow setContentView:gameView];
        [mainWindow makeKeyAndOrderFront:gameView];
        [mainWindow setLevel:windowLevel];
        [mainWindow makeFirstResponder:gameView];

    }

    // reset the tracking areas
    [gameView setTrackingAreas];
}

- (IBAction)voteWebPage:(id)sender
{
    
    
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://www.idevgames.com/content/contest_vote.php?id=10"]];
}

- (NSArray *)makeBlankHiScoresWith:(NSArray *)oldScores
{
    //int i,j;
    int j;
    NSMutableArray	*result = [NSMutableArray arrayWithCapacity:0];

    if (oldScores)	result = [NSMutableArray arrayWithArray:oldScores];

    //for (i = 0; i < 3; i++)
    while ([result count] < 8)
    {
        NSMutableArray	*scores = [NSMutableArray arrayWithCapacity:0];
        NSMutableArray	*names = [NSMutableArray arrayWithCapacity:0];
        NSMutableArray	*levels = [NSMutableArray arrayWithCapacity:0];
        for (j = 0; j < 10; j++)
        {
            [scores addObject:[NSNumber numberWithInt:100000 * (10 - j)]];
            [names addObject:[[NSBundle mainBundle]
                                localizedStringForKey:@"Vote for Garden Pests"
                                                value:nil table:nil]];
            [levels addObject:[NSNumber numberWithInt:10 - j]];
        }
        [result addObject:names];
        [result addObject:scores];
        [result addObject:levels];
    }

    return [NSArray arrayWithArray:result];
}

- (int)checkHighScores
{
    int i;
    
    // reset arrays with gameLevel
    gameNames = [hiScores objectAtIndex:0];
    gameScores = [hiScores objectAtIndex:1];
    gameLevels = [hiScores objectAtIndex:2];

    [game setHighScore:[[gameScores objectAtIndex:0] intValue]];
    
    for (i = 0; i < 10; i++)
    {
        if ([game getScore] > [[gameScores objectAtIndex:i] intValue])
        {
            /*
            for (loop = 0; loop < 10; loop++)
            {
                if ([game getScore] > [[gameScores objectAtIndex:loop] intValue])
                {
                    [gameScores	insertObject:[NSNumber numberWithInt:[game getScore]] atIndex:loop];
                    [gameScores	removeObjectAtIndex:10];
                    [gameNames	insertObject:@"new score" atIndex:loop];
                    [gameNames	removeObjectAtIndex:10];
                    [gameLevels	insertObject:[NSNumber numberWithInt:[game getLevel]] atIndex:loop];
                    [gameLevels removeObjectAtIndex:10];
                    break;
                }
            }

            [[NSUserDefaults standardUserDefaults] setObject:hiScores forKey:@"hiScores"];

            */

            return i;            
        }
    }

    return -1;
}

- (NSArray *)getHiScores
{
    return hiScores;
}

- (IBAction)resetHighScores:(id)sender
{
    [hiScores release];
    hiScores = [[self makeBlankHiScoresWith:nil] retain];
    [[NSUserDefaults standardUserDefaults] setObject:hiScores forKey:@"hiScores"];

    [gameView setNeedsDisplay:YES];
}

@end
