//
//  Game.h
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import <Foundation/Foundation.h>

@class Tile;
@class Weapon;
@class Plant;
@class ShopItem;
@class Robot;

#define GRID_WIDTH	14	// number of tiles wide
#define GRID_HEIGHT	10	// number of tiles hight

enum { GAME_TITLE, GAME_OVER, GAME_PLAYING, GAME_LEVEL_OVER, GAME_SHOP, GAME_PREFS, GAME_LIFE_OVER, GAME_MAIN_MENU, GAME_DROID_SHOP, GAME_LEVEL_INTRO, GAME_HIGH_SCORES, GAME_NEW_HIGH_SCORE };

enum { BONUS_B, BONUS_O, BONUS_N, BONUS_U, BONUS_S };

@interface Game : NSObject {
    
    NSMutableArray* tileArray;		// array of background tiles
    NSMutableArray* bulletArray;	// array of bullets
    NSMutableArray* alienArray;		// array of aliens
    NSMutableArray* explosionArray;	// array of explosions
    NSMutableArray* powerUpArray;	// array of power ups
    NSMutableArray* generatorArray;	// array of generators

    NSMutableArray* weaponArray;	// array of weapons

    NSMutableArray* plantArray;		// array of plants

    NSMutableArray* shopArray;		// array of shop items

    NSMutableArray* robotArray;		// array of robots

    bool bonusBall[BONUS_S + 1];	// array of bonus balls held
    
    int	weaponType;			// type of weapon player current has
    int score;				// current score
    int spawnRate;			// rate of new aliens being spawned
    int spawnCount;			// current spawn counter
    int ammo;				// amount of ammo left
    int shields;			// shield energy level
    int level;				// current game level
    int money;				// amount of money player has

    int gameStatus;			// current game status
    int highScore;			// game high score

    int plantheight;			// hight of the plant

    int currentShopItem;		// current item being displayed in the shop
    int currentRobotItem;		// current robot being displayed in the shop
    
    bool stabilisersInstalled;		// are the stabilisers installed?
    bool insuranceOwned;		// is an insurance policy owned?

    int novaBombs;			// number of Nova Bombs owned

    int currentRobot;			// current robot being used

    int killCount;			// number of aliens killed in current level

    int fruitCarried;			// ammount of fruit currently being carried
    int fruitCollected;			// ammount of fruit deposited into hoper

    int timeRemaining;			// ammount of time remaining for the level
    
    NSImage*	imageBackground;	// background image

    NSSound*	soundExplosion;	// explosion sound effect
    NSSound*	soundMunch;	// munch sound effect
    NSSound*	soundPuck;	// puck sound effect
    NSSound*	soundSpray;	// spray sound effect

    bool bonusLevel;		// are we in bonus level mode

    bool soundOn;		// sound effects on
    bool musicOn;		// music on

    int novaBombsUsed;		// number of nova bombs used during the level
}


- (id) init;
- (void) dealloc;

- (Tile*)getBulletAt:(int)index;
- (Tile*)getAlienAt:(int)index;
- (Tile*)getExplosionAt:(int)index;
- (Tile*)getPowerUpAt:(int)index;
- (Tile*)getTileAt:(int)index;
- (Plant*)getPlantAt:(int)index;
- (Tile*)getGeneratorAt:(int)index;
- (Robot*)getRobotAt:(int)index;

- (int)getBulletCount;
- (int)getAlienCount;
- (int)getExplosionCount;
- (int)getPowerUpCount;
- (int)getTileCount;
- (int)getPlantCount;
- (int)getGeneratorCount;
- (int)getShopItemCount;
- (int)getRobotCount;

- (Robot*)getRobot;

- (void)createBullet:(int)deltaX:(int)deltaY:(int)type;
- (void)createAlien:(int)x:(int)y:(int)deltaX:(int)deltaY:(int)type;
- (void)createExplosion:(int)x:(int)y;
- (void)createPowerUp:(int)x:(int)y:(int)deltaX:(int)deltaY:(int)type:(int)life;

- (void)createPlant:(int)x:(int)y;
- (void)createGenerator:(int)x:(int)y;

- (void)destroyBulletAt:(int)index;
- (void)destroyAlienAt:(int)index;
- (void)destroyExplosionAt:(int)index;
- (void)destroyPowerUpAt:(int)index;

- (Weapon*)getWeaponAt:(int)index;
- (int)getWeaponCount;

- (void)setWeapon:(int)newWeapon;
- (int)getWeapon;

- (void)cycleWeapons;

- (void)setScore:(int)newScore;
- (void)changeScore:(int)change;
- (int)getScore;

- (void)setHighScore:(int)newScore;
- (int)getHighScore;


- (void)setSpawnRate:(int)newRate;
- (void)changeSpawnRate:(int)change;
- (int)getSpawnRate;

- (void)setSpawnCount:(int)newCount;
- (bool)decreaseSpawnCount;

- (void)setAmmo:(int)newAmmo;
- (void)changeAmmo:(int)change;
- (int)getAmmo;

- (void)setShields:(int)newShields;
- (void)changeShields:(int)change;
- (int)getShields;

- (void)setLevel:(int)newLevel;
- (void)changeLevel:(int)change;
- (int)getLevel;

- (void)newGame;

- (void)setGameStatus:(int)newStatus;
- (int)getGameStatus;

- (void)buildLevel;
- (void)clearAllObjects;


- (NSImage*)getBackgroundImage;

- (void)createTile:(int)x:(int)y:(int)type;

- (void)setCurrentShopItem:(int)newItem;
- (void)changeCurrentShopItem:(int)change;
- (int)getCurrentShopItem;
- (ShopItem*)getShopItemAt:(int)index;

- (void)setCurrentRobotItem:(int)newItem;
- (void)changeCurrentRobotItem:(int)change;
- (int)getCurrentRobotItem;

- (void)setCurrentRobot:(int)newRobot;
- (void)changeCurrentRobot:(int)change;
- (int)getCurrentRobot;

- (bool)isShopItemOwned:(int)searchID;
- (void)removeShopItem:(int)itemID;

- (void)reloadAllWeapons;

- (void)installUpgrades;

- (void)setStabilisersInstalled:(bool)on;
- (bool)areStabilisersInstalled;

- (void)setInsuranceOwned:(bool)owned;
- (bool)isInsuranceOwned;

- (void)setMoney:(int)newMoney;
- (void)changeMoney:(int)change;
- (int)getMoney;

- (void)setNovaBombCount:(int)newCount;
- (void)changeNovaBombCount:(int)change;
- (int)getNovaBombCount;

- (void)useNovaBomb;

- (void)resetNovaBombUsedCount;
- (void)changeNovaBombUsedCount:(int)change;
- (int)getNovaBombUsedCount;

- (void)resetKillCount;
- (void)increaseKillCount;
- (int)getKillCount;

- (void)setFruitCarried:(int)newValue;
- (void)changeFruitCarried:(int)change;
- (int)getFruitCarried;

- (void)setFruitCollected:(int)newValue;
- (void)changeFruitCollected:(int)change;
- (int)getFruitCollected;

- (void)initLevel;

- (void)setTimeRemaining:(int)newTime;
- (void)changeTimeRemaining:(int)change;
- (int)getTimeRemaining;

- (void) playSound:(int)sound;

- (void)setBonusBall:(int)ball Held:(bool)held;
- (bool)isBonusBallHeld:(int)ball;

- (bool)allBonusBallsHeld;
- (void)resetBonusBalls;

- (void)setBonusLevel:(bool)bonusMode;
- (bool)isBonusLevel;

- (void)setSoundOn:(bool)on;
- (bool)isSoundOn;

- (void)setMusicOn:(bool)on;
- (bool)isMusicOn;

@end
