//
//  Game.m
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import "Game.h"
#import "Tile.h"
#import "Sprite.h"
#import "Weapon.h"
#import "Plant.h"
#import "ShopItem.h"
#import "Robot.h"

@implementation Game

- (id) init
{
    self = [super init];

    // initialise the random number seed
    srand([[NSDate date] timeIntervalSince1970]);

    // load up the sound effects
    soundExplosion = [NSSound soundNamed:@"explosion"];
    soundMunch = [NSSound soundNamed:@"munch"];
    soundPuck = [NSSound soundNamed:@"puck"];
    soundSpray = [NSSound soundNamed:@"spray"];

    robotArray = [[NSMutableArray arrayWithCapacity:0] retain];

    Robot* newRobot;

    newRobot = [[[Robot alloc] init] retain];
    [newRobot setType:0];
    [newRobot setRotation:0];
    [newRobot setPosition:200:200];
    [[newRobot getSprite] loadImageArray:@"droid three":1];
    [[newRobot getSprite] setFrames:1];
    [[newRobot getSprite] setGravityOn:NO];
    [[newRobot getSprite] setGame:self];
    [newRobot setImage:[[NSImage imageNamed:@"droid3 large"] retain]];
    [newRobot setTitle:@"SGT 1200"];
    [newRobot setSpeed:5];
    [newRobot setCarryCapacity:3];
    [newRobot setAmmoCapacity:25];
    [newRobot setDescription:@"The cheap and chearful entry level model. Voted What Garden Droid of the Year 3 years in a row."];
    [newRobot setCost:10000];


    [robotArray addObject:newRobot];

    newRobot = [[[Robot alloc] init] retain];
    [newRobot setType:0];
    [newRobot setRotation:0];
    [newRobot setPosition:200:200];
    [[newRobot getSprite] loadImageArray:@"rendered robot":2];
    [[newRobot getSprite] setFrames:2];
    [[newRobot getSprite] setGravityOn:NO];
    [[newRobot getSprite] setGame:self];
    [newRobot setImage:[[NSImage imageNamed:@"robot large"] retain]];
    [newRobot setTitle:@"HDL 1000"];
    [newRobot setSpeed:5];
    [newRobot setAmmoCapacity:50];
    [newRobot setCarryCapacity:6];
    [newRobot setDescription:@"The trusty HDL 1000 is not the fastest on the market, but its large ammo capacity makes it a favourite."];
    [newRobot setCost:15000];

    [robotArray addObject:newRobot];
    
    newRobot = [[[Robot alloc] init] retain];
    [newRobot setType:0];
    [newRobot setRotation:0];
    [newRobot setPosition:200:200];
    [[newRobot getSprite] loadImageArray:@"robot two":2];
    [[newRobot getSprite] setFrames:2];
    [[newRobot getSprite] setGravityOn:NO];
    [[newRobot getSprite] setGame:self];
    [newRobot setImage:[[NSImage imageNamed:@"robot two large"] retain]];
    [newRobot setTitle:@"Hover Bod Mk III"];
    [newRobot setSpeed:7];
    [newRobot setAmmoCapacity:25];
    [newRobot setCarryCapacity:3];
    [newRobot setDescription:@"The Mk III is very fast, yet its speed comes at the price of a low carrying capacity and a small ammo reserve."];
    [newRobot setCost:30000];


    [robotArray addObject:newRobot];

    


    [self setHighScore:0];
    [self newGame];

    [self setGameStatus:GAME_TITLE];

    [self setCurrentShopItem:0];
    [self setCurrentRobot:0];

    // load the background image
    imageBackground = [[NSImage imageNamed:@"backdrop1"] retain];

    // Do not check it with the file periodically - it won't change
    [imageBackground setDataRetained:YES];

    // Put it in memory
    [imageBackground lockFocus];
    [imageBackground unlockFocus];

    
    
    return self;
}

- (void) dealloc
{
    // release the sounds
    [soundExplosion release];
    [soundMunch release];
    [soundPuck release];
    [soundSpray release];
    
    if(robotArray)
    {
        [robotArray release];
    }
    
    if(shopArray)
    {
        [shopArray release];
    }
    
    if(generatorArray)
    {
        [generatorArray release];
    }
    
    if(weaponArray)
    {
        [weaponArray release];
    }
    
    if(tileArray)
    {
        [tileArray release];
    }


    if (bulletArray)
    {
        [bulletArray release];
    }


    if (alienArray)
    {
        [alienArray release];
    }

    if(explosionArray)
    {
        [explosionArray release];
    }

    if(powerUpArray)
    {
        [powerUpArray release];
    }
    
    [super dealloc];
}

- (void)createBullet:(int)deltaX:(int)deltaY:(int)type
{
    Tile* robot = [self getRobot];
    Tile* newBullet;
    Weapon* weapon = [self getWeaponAt:[self getWeapon]];

    newBullet = [[[Tile alloc] init] retain];
    [newBullet setType:0];
    [newBullet setRotation:0];
    [newBullet setPosition:[robot getPosition].x:[robot getPosition].y];
    [[newBullet getSprite] setSize:30:30];
    [[newBullet getSprite] setMovement:deltaX:deltaY];

    switch(type)
    {
        case 1:
            [[newBullet getSprite] setBounceOn:YES];
            break;
        case 2:
            [[newBullet getSprite] setBounceOn:NO];
            break;
    }

    [[newBullet getSprite] loadImageArray:@"spray":3];
    [[newBullet getSprite] setFrames:4];
    [[newBullet getSprite] setGame:self];


    [newBullet setLife:[weapon getLife]];

    [bulletArray addObject:newBullet];
}

- (void)createPlant:(int)x:(int)y
{
    Plant* newPlant;

    newPlant = [[[Plant alloc] init] retain];
    [newPlant setPosition:x:y];

    [plantArray addObject:newPlant];
}


- (void)createAlien:(int)x:(int)y:(int)deltaX:(int)deltaY:(int)type
{
    Tile* newAlien;

    newAlien = [[[Tile alloc] init] retain];
    [newAlien setType:type];
    [newAlien setRotation:0];
    [newAlien setPosition:x:y];
    [[newAlien getSprite] setMovement:deltaX:deltaY];
    
    [[newAlien getSprite] setBounceOn:YES];

    switch(type)
    {
        case 0:
            [[newAlien getSprite] setSize:60:20];
            [[newAlien getSprite] loadImageArray:@"crawler":2];
            [[newAlien getSprite] setFrames:2];
            break;

        case 1:
            [[newAlien getSprite] setSize:60:60];
            [[newAlien getSprite] loadImageArray:@"hunter":2];
            [[newAlien getSprite] setFrames:2];
            break;
    }
    
    [[newAlien getSprite] setGame:self];

    [newAlien setLife:-1];

    [alienArray addObject:newAlien];
}

- (void)createExplosion:(int)x:(int)y
{
    Tile* newExplosion;

    newExplosion = [[[Tile alloc] init] retain];
    [newExplosion setType:0];
    [newExplosion setRotation:0];
    [newExplosion setPosition:x:y];
    [[newExplosion getSprite] setSize:60:60];
    [[newExplosion getSprite] setMovement:0:0];
    [[newExplosion getSprite] setBounceOn:NO];
     
    [[newExplosion getSprite] loadImageArray:@"explosion":4];
    [[newExplosion getSprite] setFrames:4];

    [[newExplosion getSprite] setGame:self];

    [newExplosion setLife:7];
    [newExplosion setAnimationDelay:0];

    [explosionArray addObject:newExplosion];

    // Sound effect
    [self playSound:1];
}

- (void)createGenerator:(int)x:(int)y
{
    Tile* newGenerator;

    newGenerator = [[[Tile alloc] init] retain];
    [newGenerator setType:0];
    [newGenerator setRotation:0];
    [newGenerator setPosition:x:y];
    [[newGenerator getSprite] setSize:60:60];
    [[newGenerator getSprite] setMovement:0:0];
    [[newGenerator getSprite] setBounceOn:NO];

    [[newGenerator getSprite] loadImageArray:@"generator":1];
    [[newGenerator getSprite] setFrames:1];

    [[newGenerator getSprite] setGame:self];


    [generatorArray addObject:newGenerator];
}

- (void)createPowerUp:(int)x:(int)y:(int)deltaX:(int)deltaY:(int)type:(int)life
{
    Tile* newPowerUp;

    newPowerUp = [[[Tile alloc] init] retain];
    [newPowerUp setType:type];
    [newPowerUp setRotation:0];
    [newPowerUp setPosition:x:y];
    [[newPowerUp getSprite] setSize:60:60];
    [[newPowerUp getSprite] setMovement:deltaX:deltaY];
    [[newPowerUp getSprite] setBounceOn:YES];

    switch(type)
    {
        case 1: // refuel
            [[newPowerUp getSprite] loadImageArray:@"refuel":1];
            [[newPowerUp getSprite] setFrames:1];
            break;

        case 2: // floating thing
            [[newPowerUp getSprite] loadImageArray:@"robot":1];
            [[newPowerUp getSprite] setFrames:1];
            break;

        case 3: // fruit
            [[newPowerUp getSprite] loadImageArray:@"fruit":1];
            [[newPowerUp getSprite] setFrames:1];
            break;

        case 4: // Bonus B
            [[newPowerUp getSprite] loadImageArray:@"bonus_B":8];
            [[newPowerUp getSprite] setFrames:8];
            [newPowerUp setAnimationDelay:2];
            break;

        case 5: // Bonus O
            [[newPowerUp getSprite] loadImageArray:@"bonus_O":8];
            [[newPowerUp getSprite] setFrames:8];
            [newPowerUp setAnimationDelay:2];
            break;

        case 6: // Bonus N
            [[newPowerUp getSprite] loadImageArray:@"bonus_N":8];
            [[newPowerUp getSprite] setFrames:8];
            [newPowerUp setAnimationDelay:2];
            break;

        case 7: // Bonus U
            [[newPowerUp getSprite] loadImageArray:@"bonus_U":8];
            [[newPowerUp getSprite] setFrames:8];
            [newPowerUp setAnimationDelay:2];
            break;

        case 8: // Bonus S
            [[newPowerUp getSprite] loadImageArray:@"bonus_S":8];
            [[newPowerUp getSprite] setFrames:8];
            [newPowerUp setAnimationDelay:2];
            break;

        case 9: // fruit hopper
            [[newPowerUp getSprite] loadImageArray:@"hopper":3];
            [[newPowerUp getSprite] setFrames:4];
            break;

        case 10: // point bonus
            [[newPowerUp getSprite] loadImageArray:@"bonus":8];
            [[newPowerUp getSprite] setFrames:8];
            [newPowerUp setAnimationDelay:1];
            break;
    }

    [[newPowerUp getSprite] setGame:self];
    
    [newPowerUp setLife:life];

    [powerUpArray addObject:newPowerUp];
}

- (Robot*)getRobot
{
    return [self getRobotAt:[self getCurrentRobot]];
}

- (Weapon*)getWeaponAt:(int)index
{
    Weapon* thisWeapon;

    thisWeapon = [weaponArray objectAtIndex:index];

    return thisWeapon;
}


- (int)getWeaponCount
{
    return [weaponArray count];
}

- (void)cycleWeapons
{
    weaponType++;

    if(weaponType == [self getWeaponCount])
    {
        weaponType = 0;
    }
}

- (int)getShopItemCount
{
    return [shopArray count];
}

- (ShopItem*)getShopItemAt:(int)index
{
    ShopItem* thisItem;

    thisItem = [shopArray objectAtIndex:index];

    return thisItem;
}

- (Tile*)getAlienAt:(int)index
{
    Tile* thisAlien;

    thisAlien = [alienArray objectAtIndex:index];

    return thisAlien;
}

- (int)getAlienCount
{
    return [alienArray count];
}

- (Tile*)getExplosionAt:(int)index
{
    Tile* thisExplosion;

    thisExplosion = [explosionArray objectAtIndex:index];

    return thisExplosion;
}

- (int)getExplosionCount
{
    return [explosionArray count];
}

- (Tile*)getPowerUpAt:(int)index
{
    Tile* thisPowerUp;

    thisPowerUp = [powerUpArray objectAtIndex:index];

    return thisPowerUp;
}

- (int)getPowerUpCount
{
    return [powerUpArray count];
}

- (void)destroyAlienAt:(int)index
{
    [alienArray removeObjectAtIndex:index];
}

- (void)destroyBulletAt:(int)index
{
    [bulletArray removeObjectAtIndex:index];
}

- (void)destroyExplosionAt:(int)index
{
    [explosionArray removeObjectAtIndex:index];
}

- (void)destroyPowerUpAt:(int)index
{
    [powerUpArray removeObjectAtIndex:index];
}


- (Tile*)getBulletAt:(int)index
{
    Tile* thisBullet;

    thisBullet = [bulletArray objectAtIndex:index];

    return thisBullet;
}

- (int)getRobotCount
{
    return [robotArray count];
}

- (Robot*)getRobotAt:(int)index
{
    Robot* thisRobot;

    thisRobot = [robotArray objectAtIndex:index];

    return thisRobot;
}

- (int)getPlantCount
{
    return [plantArray count];
}

- (Plant*)getPlantAt:(int)index
{
    Plant* thisPlant;

    thisPlant = [plantArray objectAtIndex:index];

    return thisPlant;
}

- (Tile*)getGeneratorAt:(int)index
{
    Tile* thisGenerator;

    thisGenerator = [generatorArray objectAtIndex:index];

    return thisGenerator;
}

- (int)getGeneratorCount
{
    return [generatorArray count];
}

- (int)getBulletCount
{
    return [bulletArray count];
}

- (Tile*)getTileAt:(int)index
{
    Tile* thisTile;

    thisTile = [tileArray objectAtIndex:index];

    return thisTile;
}

-(int)getTileCount
{
    return [tileArray count];
}

- (void)setWeapon:(int)newWeapon
{
    weaponType = newWeapon;
}

- (int)getWeapon
{
    return weaponType;
}

- (void)setScore:(int)newScore
{
    score = newScore;
}

- (void)changeScore:(int)change
{
    score += change;
}

- (int)getScore
{
    return score;
}

- (void)setHighScore:(int)newScore
{
    highScore = newScore;
}

- (int)getHighScore
{
    return highScore;
}

- (void)setSpawnRate:(int)newRate
{
    spawnRate = newRate;
}

- (void)changeSpawnRate:(int)change
{
    spawnRate += change;
}

- (int)getSpawnRate
{
    return spawnRate;
}

- (void)setSpawnCount:(int)newCount
{
    spawnCount = newCount;
}

- (bool)decreaseSpawnCount
{
    if(spawnCount > 0)
    {
        spawnCount--;
        return NO;
    }

    return YES;
}

- (void)setAmmo:(int)newAmmo
{
    ammo = newAmmo;
}

- (void)changeAmmo:(int)change
{
    ammo += change;

    // max ammo is 100
    if(ammo > 100)
    {
        ammo = 100;
    }

    if(ammo < 0)
    {
        ammo = 0;
    }
}

- (int)getAmmo
{
    return ammo;
}

- (void)setShields:(int)newShields
{
    shields = newShields;
}

- (void)changeShields:(int)change
{
    shields += change;

    // max shields is 100
    if(shields > 100)
    {
        shields = 100;
    }

    if(shields < 0)
    {
        shields = 0;
    }
}

- (int)getShields
{
    return shields;
}

- (void)clearAllObjects
{
    if(weaponArray)
    {
        [weaponArray release];
    }

    weaponArray = [[NSMutableArray arrayWithCapacity:0] retain];
    
    if (generatorArray)
    {
        [generatorArray release];
    }

    generatorArray = [[NSMutableArray arrayWithCapacity:0] retain];
    
    if (tileArray)
    {
        [tileArray release];
    }

    tileArray = [[NSMutableArray arrayWithCapacity:0] retain];

    if (bulletArray)
    {
        [bulletArray release];
    }

    bulletArray = [[NSMutableArray arrayWithCapacity:0] retain];

    if(alienArray)
    {
        [alienArray release];
    }

    alienArray = [[NSMutableArray arrayWithCapacity:0] retain];

    if(explosionArray)
    {
        [explosionArray release];
    }

    explosionArray = [[NSMutableArray arrayWithCapacity:0] retain];

    if(powerUpArray)
    {
        [powerUpArray release];
    }

    powerUpArray = [[NSMutableArray arrayWithCapacity:0] retain];

    if(plantArray)
    {
        [plantArray release];
    }

    // create plant array
    plantArray = [[NSMutableArray arrayWithCapacity:0] retain];
}

- (void)newGame
{
    [self clearAllObjects];

    // create the shop array
    if(shopArray)
    {
        [shopArray release];
    }

    shopArray = [[NSMutableArray arrayWithCapacity:0] retain];

    // stock the shop
    ShopItem* newItem;
    newItem = [[[ShopItem alloc] init] retain];
    [newItem configure:@"Stablisers":10000:ITEM_STABLISERS];
    [newItem setDescription:@"The stabilsers prevent the droid drifting when told to stop moving."];
    [newItem setImage:[[NSImage imageNamed:@"shop_stablisers"] retain]];
    [shopArray addObject:newItem];

    newItem = [[[ShopItem alloc] init] retain];
    [newItem configure:@"Double Fire":5000:ITEM_DOUBLE_FIRE];
    [newItem setDescription:@"Fires forward and backwards."];
    [newItem setImage:[[NSImage imageNamed:@"shop_double_fire"] retain]];
    [shopArray addObject:newItem];

    newItem = [[[ShopItem alloc] init] retain];
    [newItem configure:@"Fourway Fire":10000:ITEM_FOURWAY_FIRE];
    [newItem setDescription:@"Fires forward, backward, up and down."];
    [newItem setImage:[[NSImage imageNamed:@"shop_four_fire"] retain]];
    [shopArray addObject:newItem];

    newItem = [[[ShopItem alloc] init] retain];
    [newItem configure:@"Nova Bomb":10000:ITEM_NOVA_BOMB];
    [newItem setDescription:@"The Nova Bomb wipes out all the pests. Beware that the toxic fallout reduces the value of the Galactic Strawberries harvested."];
    [newItem setImage:[[NSImage imageNamed:@"shop_nova_bomb"] retain]];
    [shopArray addObject:newItem];

    newItem = [[[ShopItem alloc] init] retain];
    [newItem configure:@"Insurance":5000:ITEM_INSURANCE];
    [newItem setDescription:@"New for Old Garden Droid insurance provides you with a replacement Garden Droid if your droid should get destroyed."];
    [newItem setImage:[[NSImage imageNamed:@"shop_insurance"] retain]];
    [shopArray addObject:newItem];


    [self setShields:0];
    [self setWeapon:0];
    [self setScore:0];    [self setLevel:1];
    
    [self setAmmo:100];
    [self setMoney:10000];
    [self setNovaBombCount:0];

    [self setCurrentRobotItem:0];
    [self setCurrentRobot:-1];

    // clear the bonus balls
    [self setBonusBall:BONUS_B
                  Held:NO];

    [self setBonusBall:BONUS_O
                  Held:NO];

    [self setBonusBall:BONUS_N
                  Held:NO];

    [self setBonusBall:BONUS_U
                  Held:NO];

    [self setBonusBall:BONUS_S
                  Held:NO];

    [self setBonusLevel:NO];
    

    // set the plant height
    plantheight = 0;
    
}


- (void)setGameStatus:(int)newStatus
{
    gameStatus = newStatus;
}

- (int)getGameStatus
{
    return gameStatus;
}

- (void)buildLevel
{
    int row, column;
    
    if(tileArray)
    {
        [tileArray release];
    }
    
    // create background tile array
    tileArray = [[NSMutableArray arrayWithCapacity:0] retain];

    //[self setBonusLevel:YES];
    
    if([self isBonusLevel])
    {
        // create the walls and floor
        for(row = 0; row < GRID_WIDTH; row++)
        {
            for(column = 0; column < GRID_HEIGHT; column++)
            {
                if(row == 0 || row == GRID_WIDTH - 1 || column == 0 || column == GRID_HEIGHT - 1)
                {
                    [self createTile:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                }
            }
        }

        for(row = 3; row < GRID_WIDTH - 3; row++)
        {
            [self createTile:(row * TILE_WIDTH) - 15:3*TILE_HEIGHT:0];
            [self createTile:(row * TILE_WIDTH) - 15:(GRID_HEIGHT - 4)*TILE_HEIGHT:0];
        }

        // add the bonus fruit
        for(row = 1; row < GRID_WIDTH - 1; row++)
        {
            for(column = 1; column < GRID_HEIGHT - 1; column++)
            {
                if(column != 3 && column != GRID_HEIGHT - 4)
                {
                    [self createPowerUp:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0:0:10:-1];
                }
                else if(row < 3 || row > GRID_WIDTH - 4)
                {
                    [self createPowerUp:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0:0:10:-1];

                }
            }
        }

                
        // set start position
        [[self getRobot] setPosition:(1*TILE_WIDTH) - 15:(4*TILE_HEIGHT)];
        [[[self getRobot] getSprite] setMovement:0:0];

        [self setTimeRemaining:2500 / 4];
    
    }
    else
    {
        switch(([self getLevel] - 1) % 6)
        {
            case 4:
            {

                // create the walls and floor
                for(row = 0; row < GRID_WIDTH; row++)
                {
                    for(column = 0; column < GRID_HEIGHT; column++)
                    {
                        if(row == 0 || row == GRID_WIDTH - 1 || column == 0 || column == GRID_HEIGHT - 1)
                        {
                            [self createTile:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                        }
                    }
                }

                // create shelves

                for(column = 1; column < 4; column++)
                {
                    for(row = 1; row < 5 - column; row++)
                    {
                        [self createTile:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                        [self createTile:((GRID_WIDTH - 1 - row)*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                    }
                }


                // create shelves
                [self createTile:(5*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];
                [self createTile:(6*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];
                [self createTile:(7*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];
                [self createTile:(8*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];

                // create ammo refill point
                [self createPowerUp:(5*TILE_WIDTH) - 15:(7*TILE_HEIGHT) - 5:0:0:1:-1];

                // create fruit drop off point
                [self createPowerUp:(8*TILE_WIDTH) - 15:(7*TILE_HEIGHT):0:0:9:-1];

                // create generators
                [self createGenerator:(3 * TILE_WIDTH) - 15:2*TILE_HEIGHT];
                [self createGenerator:(10 * TILE_WIDTH) - 15:2*TILE_HEIGHT];

                // plant the seeds
                [self createPlant:1*TILE_WIDTH:(4*TILE_HEIGHT) - 10];
                [self createPlant:(GRID_WIDTH - 2) *TILE_WIDTH:(4*TILE_HEIGHT) - 10];

                // set start position
                [[self getRobot] setPosition:(1*TILE_WIDTH) - 15:(4*TILE_HEIGHT)];
                [[[self getRobot] getSprite] setMovement:0:0];
            }


                break;

                
            case 3:
            {

                // create the walls and floor
                for(row = 0; row < GRID_WIDTH; row++)
                {
                    for(column = 0; column < GRID_HEIGHT; column++)
                    {
                        if(row == 0 || row == GRID_WIDTH - 1 || column == 0 || column == GRID_HEIGHT - 1)
                        {
                            [self createTile:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                        }
                    }
                }

                // create shelves

                for(row = 1; row < 4; row++)
                {
                    [self createTile:(row*TILE_WIDTH) - 15:3*TILE_HEIGHT:0];
                    [self createTile:((GRID_WIDTH - 1 - row)*TILE_WIDTH) - 15:3*TILE_HEIGHT:0];
                }


                // create ammo refill point
                [self createPowerUp:(1*TILE_WIDTH) - 15:(1*TILE_HEIGHT) - 5:0:0:1:-1];

                // create fruit drop off point
                [self createPowerUp:((GRID_WIDTH - 2)*TILE_WIDTH) - 15:(1*TILE_HEIGHT):0:0:9:-1];

                // create generators
                [self createGenerator:(5 * TILE_WIDTH) - 15:1*TILE_HEIGHT];
                [self createGenerator:(8 * TILE_WIDTH) - 15:1*TILE_HEIGHT];

                // plant the seeds
                [self createPlant:1*TILE_WIDTH:(4*TILE_HEIGHT) - 10];
                [self createPlant:(GRID_WIDTH - 2) *TILE_WIDTH:(4*TILE_HEIGHT) - 10];

                // set start position
                [[self getRobot] setPosition:(1*TILE_WIDTH) - 15:(4*TILE_HEIGHT)];
                [[[self getRobot] getSprite] setMovement:0:0];
            }


                break;
                

            case 5:
            {

                // create the walls and floor
                for(row = 0; row < GRID_WIDTH; row++)
                {
                    for(column = 0; column < GRID_HEIGHT; column++)
                    {
                        if(row == 0 || row == GRID_WIDTH - 1 || column == 0 || column == GRID_HEIGHT - 1)
                        {
                            [self createTile:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                        }
                    }
                }

                for(column = 1; column < GRID_HEIGHT - 3; column++)
                {
                    [self createTile:(4 *TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                    [self createTile:(9 *TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                }
              
                
                // create ammo refill point
                [self createPowerUp:(5*TILE_WIDTH) - 15:(1*TILE_HEIGHT) - 5:0:0:1:-1];

                // create fruit drop off point
                [self createPowerUp:(8*TILE_WIDTH) - 15:(1*TILE_HEIGHT):0:0:9:-1];

                // create generators
                [self createGenerator:(4 * TILE_WIDTH) - 15:7*TILE_HEIGHT];
                [self createGenerator:(9 * TILE_WIDTH) - 15:7*TILE_HEIGHT];

                // plant the seeds
                [self createPlant:(2*TILE_WIDTH):50];
                [self createPlant:(11*TILE_WIDTH):50];


                // set start position
                [[self getRobot] setPosition:(6*TILE_WIDTH) - 15:(4*TILE_HEIGHT)];
                [[[self getRobot] getSprite] setMovement:0:0];
            }

                
                break;

            case 0:
            {

                // create the walls and floor
                for(row = 0; row < GRID_WIDTH; row++)
                {
                    for(column = 0; column < GRID_HEIGHT; column++)
                    {
                        if(row == 0 || row == GRID_WIDTH - 1 || column == 0 || column == GRID_HEIGHT - 1)
                        {
                            [self createTile:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                        }
                    }
                }

                // create shelves
                [self createTile:(1*TILE_WIDTH) - 15:3*TILE_HEIGHT:0];
                [self createTile:(1*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];
                [self createTile:((GRID_WIDTH - 2)*TILE_WIDTH) - 15:3*TILE_HEIGHT:0];
                [self createTile:((GRID_WIDTH - 2)*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];


                // create ammo refill point
                [self createPowerUp:(1*TILE_WIDTH) - 15:(4*TILE_HEIGHT) - 5:0:0:1:-1];

                // create fruit drop off point
                [self createPowerUp:((GRID_WIDTH - 2)*TILE_WIDTH) - 15:(4*TILE_HEIGHT):0:0:9:-1];

                // create generators
                [self createGenerator:((1)*TILE_WIDTH) - 15:7*TILE_HEIGHT];
                [self createGenerator:((GRID_WIDTH - 2)*TILE_WIDTH) - 15:7*TILE_HEIGHT];
                [self createGenerator:((1)*TILE_WIDTH) - 15:1*TILE_HEIGHT];
                [self createGenerator:((GRID_WIDTH - 2)*TILE_WIDTH) - 15:1*TILE_HEIGHT];

                // plant the seeds
                [self createPlant:200:50];
                [self createPlant:400:50];
                [self createPlant:600:50];

                // set start position
                [[self getRobot] setPosition:(1*TILE_WIDTH) - 15:(4*TILE_HEIGHT)];
                [[[self getRobot] getSprite] setMovement:0:0];
            }


                break;
                
            case 1:
            {
                // create the walls and floor
                for(row = 0; row < GRID_WIDTH; row++)
                {
                    for(column = 0; column < GRID_HEIGHT; column++)
                    {
                        if(row == 0 || row == GRID_WIDTH - 1 || column == 0 || column == GRID_HEIGHT - 1)
                        {
                            [self createTile:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                        }
                    }
                }

                // create shelves
                [self createTile:(6*TILE_WIDTH) + (TILE_WIDTH / 2) - 15:5*TILE_HEIGHT:0];
                [self createTile:(5*TILE_WIDTH) + (TILE_WIDTH / 2) - 15:4*TILE_HEIGHT:0];
                [self createTile:(7*TILE_WIDTH) + (TILE_WIDTH / 2) - 15:4*TILE_HEIGHT:0];
                [self createTile:(4*TILE_WIDTH) + (TILE_WIDTH / 2) - 15:3*TILE_HEIGHT:0];
                [self createTile:(8*TILE_WIDTH) + (TILE_WIDTH / 2) - 15:3*TILE_HEIGHT:0];
                //[self createTile:(6*TILE_WIDTH) - 15:5*TILE_HEIGHT:0];
                //[self createTile:(7*TILE_WIDTH) - 15:5*TILE_HEIGHT:0];
                //[self createTile:(8*TILE_WIDTH) - 15:5*TILE_HEIGHT:0];

                // create ammo refill point
                [self createPowerUp:(6*TILE_WIDTH) + (TILE_WIDTH / 2) - 15:(6*TILE_HEIGHT) - 5:0:0:1:-1];

                // create fruit drop off point
                [self createPowerUp:(6*TILE_WIDTH) + (TILE_WIDTH / 2)- 15:(1*TILE_HEIGHT):0:0:9:-1];

                // create generators
                [self createGenerator:(1*TILE_WIDTH) - 15:1*TILE_HEIGHT];
                [self createGenerator:((GRID_WIDTH -2 )*TILE_WIDTH) - 15:1*TILE_HEIGHT];

                [self createGenerator:(1*TILE_WIDTH) - 15:7*TILE_HEIGHT];
                [self createGenerator:((GRID_WIDTH -2 )*TILE_WIDTH) - 15:7*TILE_HEIGHT];

                // plant seeds
                [self createPlant:(4*TILE_WIDTH) + (TILE_WIDTH / 2):(4*TILE_HEIGHT) - 10];
                [self createPlant:(8*TILE_WIDTH) + (TILE_WIDTH / 2):(4*TILE_HEIGHT) - 10];

                // set start position
                [[self getRobot] setPosition:(6*TILE_WIDTH) - 15:(6*TILE_HEIGHT)];
                [[[self getRobot] getSprite] setMovement:0:0];
            }

                break;


            case 2:
            {
                // create the walls and floor
                for(row = 0; row < GRID_WIDTH; row++)
                {
                    for(column = 0; column < GRID_HEIGHT; column++)
                    {
                        if(row == 0 || row == GRID_WIDTH - 1 || column == 0 || column == GRID_HEIGHT - 1)
                        {
                            [self createTile:(row*TILE_WIDTH) - 15:column*TILE_HEIGHT:0];
                        }
                    }
                }

                // create shelves
                [self createTile:(5*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];
                [self createTile:(6*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];
                [self createTile:(7*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];
                [self createTile:(8*TILE_WIDTH) - 15:6*TILE_HEIGHT:0];

                // create ammo refill point
                [self createPowerUp:(5*TILE_WIDTH) - 15:(7*TILE_HEIGHT) - 5:0:0:1:-1];

                // create fruit drop off point
                [self createPowerUp:(8*TILE_WIDTH) - 15:(7*TILE_HEIGHT):0:0:9:-1];
                
                // create generators
                [self createGenerator:(1*TILE_WIDTH) - 15:1*TILE_HEIGHT];
                [self createGenerator:((GRID_WIDTH -2 )*TILE_WIDTH) - 15:1*TILE_HEIGHT];

                // plant seeds
                [self createPlant:200:50];
                [self createPlant:400:50];
                [self createPlant:600:50];

                // set start position
                [[self getRobot] setPosition:(5*TILE_WIDTH) - 15:(7*TILE_HEIGHT)];
                [[[self getRobot] getSprite] setMovement:0:0];
            }

                
                break;
        }

        [self setTimeRemaining:2500 / 2];

        int rate;

        rate = 40 - ((([self getLevel] - 1) / 6) * 10);

        if(rate < 10)
        {
            rate = 10;
        }
        
        [self setSpawnRate:rate]; // was 50
        [self setSpawnCount:[self getSpawnRate]];

    }

}

- (void)createTile:(int)x:(int)y:(int)type
{
    Tile* newTile;
    newTile = [[[Tile alloc] init] retain];

    [newTile setRotation:(rand()%4) * 90];
    [newTile setPosition:x:y];

    [newTile setType:type];
    [[newTile getSprite] setGame:self];

    switch(type)
    {
        case 0: // Block
            [[newTile getSprite] loadImageArray:@"ivy stone":1];
            [[newTile getSprite] setFrames:1];
            break;

        case 1: // Generator
            [[newTile getSprite] loadImageArray:@"generator":1];
            [[newTile getSprite] setFrames:1];
            break;
    }

    [tileArray addObject:newTile];
}

- (NSImage*)getBackgroundImage
{
    return imageBackground;
}

- (void)setLevel:(int)newLevel
{
    level = newLevel;
}

- (void)changeLevel:(int)change
{
    level += change;
}

- (int)getLevel
{
    return level;
}

- (void)setCurrentShopItem:(int)newItem
{
    currentShopItem = newItem;
}

- (void)changeCurrentShopItem:(int)change
{
    currentShopItem += change;

    if(currentShopItem < 0)
    {
        [self setCurrentShopItem:[self getShopItemCount] - 1];
    }

    if(currentShopItem == [self getShopItemCount])
    {
        [self setCurrentShopItem:0];
    }
}

- (void)setCurrentRobot:(int)newRobot
{
    currentRobot = newRobot;
}

- (void)changeCurrentRobot:(int)change
{
    currentRobot += change;

    if(currentRobot < 0)
    {
        [self setCurrentRobot:[self getRobotCount] - 1];
    }

    if(currentRobot == [self getRobotCount])
    {
        [self setCurrentRobot:0];
    }
}

- (int)getCurrentRobot
{
    return currentRobot;
}

- (int)getCurrentShopItem
{
    return currentShopItem;
}

- (void)setCurrentRobotItem:(int)newItem
{
    currentRobotItem = newItem;
}

- (void)changeCurrentRobotItem:(int)change
{
    currentRobotItem += change;

    if(currentRobotItem < 0)
    {
        [self setCurrentRobotItem:[self getRobotCount] -1];
    }

    if(currentRobotItem == [self getRobotCount])
    {
        [self setCurrentRobotItem:0];
    }
}

- (int)getCurrentRobotItem
{
    return currentRobotItem;
}


- (bool)isShopItemOwned:(int)searchID
{
    int loop;
    ShopItem* item;

    for(loop = 0; loop < [self getShopItemCount]; loop++)
    {
        item = [self getShopItemAt:loop];

        if([item getID] == searchID)
        {
            return [item isBought];
        }
    }

    return NO;
}

// remove item from own possession
- (void)removeShopItem:(int)itemID
{
    int loop;
    ShopItem* item;

    for(loop = 0; loop < [self getShopItemCount]; loop++)
    {
        item = [self getShopItemAt:loop];

        if([item getID] == itemID)
        {
            [item setBought:NO];
            return;
        }
    }    
}

- (void)reloadAllWeapons
{
    int loop;
    
    for(loop = 0; loop < [self getWeaponCount]; loop++)
    {
        [[self getWeaponAt:loop] setAmmo:[[self getWeaponAt:loop] getMaxAmmo]];
    }
}

- (void)installUpgrades
{
    // add default weapon
    Weapon* newWeapon;
    newWeapon = [[[Weapon alloc] init] retain];
    [newWeapon configure:@"Front Bounce":NO:NO:YES:NO:YES:50];
    [newWeapon setMaxAmmo:[[self getRobot] getAmmoCapacity]];
    [newWeapon setAmmo:[[self getRobot] getAmmoCapacity]];
    [weaponArray addObject:newWeapon];

    [self setWeapon:0];


    
    [self setStabilisersInstalled:[self isShopItemOwned:ITEM_STABLISERS]];

    [self setInsuranceOwned:[self isShopItemOwned:ITEM_INSURANCE]];

    if([self isShopItemOwned:ITEM_DOUBLE_FIRE])
    {
        newWeapon = [[[Weapon alloc] init] retain];
        [newWeapon configure:@"Double Fire":NO:NO:YES:YES:YES:25];
        [newWeapon setMaxAmmo:[[self getRobot] getAmmoCapacity]];
        [newWeapon setAmmo:[[self getRobot] getAmmoCapacity]];
        [weaponArray addObject:newWeapon];
    }

    if([self isShopItemOwned:ITEM_FOURWAY_FIRE])
    {
        newWeapon = [[[Weapon alloc] init] retain];
        [newWeapon configure:@"Fourway Fire":YES:YES:YES:YES:YES:25];
        [newWeapon setMaxAmmo:[[self getRobot] getAmmoCapacity]];
        [newWeapon setAmmo:[[self getRobot] getAmmoCapacity]];
        [weaponArray addObject:newWeapon];
    }

}

- (void)setStabilisersInstalled:(bool)on
{
    stabilisersInstalled = on;
}
    
- (bool)areStabilisersInstalled
{
    return stabilisersInstalled;
}

- (void)setInsuranceOwned:(bool)owned
{
    insuranceOwned = owned;
}

- (bool)isInsuranceOwned
{
    return insuranceOwned;
}

- (void)setMoney:(int)newMoney
{
    money = newMoney;
}

- (void)changeMoney:(int)change
{
    money += change;

    if(money < 0)
    {
        money = 0;
    }
}

- (int)getMoney
{
    return money;
}


- (void)setNovaBombCount:(int)newCount
{
    novaBombs = newCount;
}

- (void)changeNovaBombCount:(int)change
{
    novaBombs += change;

    if(novaBombs < 0)
    {
        novaBombs = 0;
    }
}

- (int)getNovaBombCount
{
    return novaBombs;
}

- (void)resetNovaBombUsedCount
{
    novaBombsUsed = 0;
}

- (void)changeNovaBombUsedCount:(int)change
{
    novaBombsUsed += change;
}

- (int)getNovaBombUsedCount
{
    return novaBombsUsed;
}

- (void)useNovaBomb
{
    int loopAlien;
    Tile* alien;
    
    // loop through all the aliens and blow them all up :-)
    
    for(loopAlien = 0; loopAlien < [self getAlienCount]; loopAlien++)
    {
        alien = [self getAlienAt:loopAlien];
        [self createExplosion:[alien getPosition].x:[alien getPosition].y];
        [self destroyAlienAt:loopAlien];
        
        loopAlien--;

        [self increaseKillCount];
        [self changeScore:100];
    }

    // decrease Nova Bomb supply
    [self changeNovaBombCount:-1];

    // increase Nova Bomb used count for working out environmental damage
    [self changeNovaBombUsedCount:1];
    
}

- (void)resetKillCount
{
    killCount = 0;
}

- (void)increaseKillCount
{
    killCount++;
}

- (int)getKillCount
{
    return killCount;
}


- (void)setFruitCarried:(int)newValue
{
    fruitCarried = newValue;
}

- (void)changeFruitCarried:(int)change
{
    fruitCarried += change;

    if(fruitCarried < 0)
    {
        fruitCarried = 0;
    }
}

- (int)getFruitCarried
{
    return fruitCarried;
}

- (void)setFruitCollected:(int)newValue
{
    fruitCollected = newValue;
}

- (void)changeFruitCollected:(int)change
{
    fruitCollected += change;

    if(fruitCollected < 0)
    {
        fruitCollected = 0;
    }
}

- (int)getFruitCollected
{
    return fruitCollected;
}


- (void)initLevel
{
    [self buildLevel];
    [self setFruitCarried:0];
    [self setFruitCollected:0];
    [self resetKillCount];
    [self installUpgrades];
    [self reloadAllWeapons];
    [self setShields:100];
    [self resetNovaBombUsedCount];
}

- (void)setTimeRemaining:(int)newTime
{
    timeRemaining = newTime;
}

- (void)changeTimeRemaining:(int)change
{
    timeRemaining += change;
}

- (int)getTimeRemaining
{
    return timeRemaining;
}


- (void) playSound:(int)sound
{
    
    if(![self isSoundOn])
    {
        return;
    }
    
    
    switch(sound)
    {
        case 1: // explosion
            [[[soundExplosion copy] autorelease] play];
            break;

        case 2: // munch
            [[[soundMunch copy] autorelease] play];
            break;

        case 3: // puck
            [[[soundPuck copy] autorelease] play];
            break;

        case 4: // spray
            [[[soundSpray copy] autorelease] play];
            break;
    }
}

- (void)setBonusBall:(int)ball Held:(bool)held
{
    bonusBall[ball] = held;
}

- (bool)isBonusBallHeld:(int)ball
{
    return bonusBall[ball];
}

- (bool)allBonusBallsHeld
{
    int loop;

    // check to see if all the bonus balls are held
    for(loop = BONUS_B; loop < BONUS_S + 1; loop++)
    {
        if([self isBonusBallHeld:loop] == NO)
        {
            return NO;
        }
    }

    [self resetBonusBalls];

    [self setBonusLevel:YES];

    return YES;

    //[self setGameStatus:GAME_LEVEL_INTRO];
    //[self clearAllObjects];
    //[self initLevel];
}

- (void)resetBonusBalls
{
    int loop;

    for(loop = BONUS_B; loop < BONUS_S + 1; loop++)
    {
        [self setBonusBall:loop
                      Held:NO];
    }
    
}

- (void)setBonusLevel:(bool)bonusMode
{
    bonusLevel = bonusMode;
}

- (bool)isBonusLevel
{
    return bonusLevel;
}

- (void)setSoundOn:(bool)on
{
    soundOn = on;

    // save the new music on / off state to the user defaults
    [[NSUserDefaults standardUserDefaults] setBool:soundOn forKey:@"SoundOn"];
}

- (bool)isSoundOn
{
    return soundOn;
}

- (void)setMusicOn:(bool)on
{
    musicOn = on;

    // save the new music on / off state to the user defaults
    [[NSUserDefaults standardUserDefaults] setBool:musicOn forKey:@"MusicOn"];
}

- (bool)isMusicOn
{
    return musicOn;
}

@end
