//
//  GameView.h
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import <Cocoa/Cocoa.h>

@class Game;
@class Controller;

enum { KEY_LEFT, KEY_RIGHT, KEY_UP, KEY_DOWN, KEY_FIRE, KEY_CYCLE_WEAPONS, KEY_NOVA_BOMB, KEY_PAUSE, KEY_LAST};

enum { MENU_NONE, MENU_DROID_SHOP, MENU_UPGRADES, MENU_DOMES, BUTTON_NEXT, BUTTON_PREVIOUS, BUTTON_BUY, BUTTON_LEAVE_SHOP, BUTTON_OPTIONS, BUTTON_PLAY, BUTTON_DONE, BUTTON_CONTINUE, BUTTON_UDEVGAME, BUTTON_HIGH_SCORES, BUTTON_MUSIC, BUTTON_SOUND, BUTTONS_LAST };

@interface GameView : NSView
{
    IBOutlet Controller		*controller;
    NSColor	*backgroundColour;	// screen colour
    Game	*game;
    NSTimer	*timer;

    // fps counter code
    NSTimeInterval elapsed;
    NSTimeInterval lastFrame;
    unsigned int frameCount;
    NSString* sFrameRate;
    bool	bLocked;
    bool	paused;

    bool	keymap[63232 + 1000];	// keyboard map
    bool	keypressed[KEY_LAST];	// keys pressed
    int		keys[KEY_LAST];		// key for each command
    int		buttons[BUTTONS_LAST];	// tracking number for each tracking rectangle

    bool	reprogramModeOn;	// keys reprogramming mode on
    int		currentReprogramKey;	// current key being reprogrammed

    int		currentMenuItem;	// currently selected menu item

    int		currentHighScoreRanking;	// ranking of current highscore

    NSPoint	lastMousePoint;		// last known mouse point used for tracking rects
}

// Standard view create / free methods
- (id) initWithFrame:(NSRect)frame;
- (void) dealloc;

- (void) setGame:(Game*) agame;
- (Game *) game;


- (void) setController:(Controller*) acontroller;

- (void) runTimer;

- (void)drawPowerBar:(int)x:(int)y:(float)length:(int)height:(float)value:(float)maxValue:(float)red:(float)green:(float)blue;

- (void)processKeyboard;


- (void)drawGameDisplay;
- (void)drawTitleDisplay;
- (void)drawGameOverDisplay;
- (void)drawLevelOverDisplay;
- (void)drawLifeOverDisplay;
- (void)drawShopDisplay;
- (void)drawDroidShopDisplay;
- (void)drawPrefsDisplay;
- (void)drawMainMenuDisplay;
- (void)drawLevelIntroDisplay;
- (void)drawHighScoresDisplay;
- (void)drawNewHighScoreDisplay;

- (void)displayText:(NSString*)text AtX:(int)x AtY:(int)y FontSize:(int)fontSize;
- (void)displayText:(NSString*)text:(int)x:(int)y:(int)width:(int)height;

- (NSString*)displayKey:(int)key;

- (void)setTrackingAreas;

- (void)createTrackingArea:(NSRect)rect AndItem:(int)item;

@end
