//
//  GameView.m
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import "GameView.h"
#import "Game.h"
#import "Tile.h"
#import "Sprite.h"
#import "Weapon.h"
#import "Plant.h"
#import "ShopItem.h"
#import "Robot.h"
#import "Controller.h"

@implementation GameView

- (id)initWithFrame:(NSRect)frameRect
{
    bool settingsExist = NO;	// do we already have settings
    
    [super initWithFrame:frameRect];

    // load up the keys
    settingsExist = [[NSUserDefaults standardUserDefaults] boolForKey:@"settings"];

    if(settingsExist)
    {
        keys[KEY_UP] = [[NSUserDefaults standardUserDefaults] integerForKey:@"key_up"];
        keys[KEY_DOWN] = [[NSUserDefaults standardUserDefaults] integerForKey:@"key_down"];
        keys[KEY_LEFT] = [[NSUserDefaults standardUserDefaults] integerForKey:@"key_left"];
        keys[KEY_RIGHT] = [[NSUserDefaults standardUserDefaults] integerForKey:@"key_right"];
        keys[KEY_FIRE] = [[NSUserDefaults standardUserDefaults] integerForKey:@"key_fire"];
        keys[KEY_CYCLE_WEAPONS] = [[NSUserDefaults standardUserDefaults] integerForKey:@"key_cycle_weapons"];
        keys[KEY_NOVA_BOMB] = [[NSUserDefaults standardUserDefaults] integerForKey:@"key_nova_bomb"];
        keys[KEY_PAUSE] = [[NSUserDefaults standardUserDefaults] integerForKey:@"key_pause"];
    }
    else
    {
        keys[KEY_UP] = (int)'p';
        keys[KEY_DOWN] = (int)'l';
        keys[KEY_LEFT] = (int)'z';
        keys[KEY_RIGHT] = (int)'x';
        keys[KEY_FIRE] = ' ';
        keys[KEY_CYCLE_WEAPONS] = '1';
        keys[KEY_NOVA_BOMB] = '2';
        keys[KEY_PAUSE] = 't';

        // save the settings exist flag
        [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"settings"];

        // save the keys
        [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_UP] forKey:@"key_up"];
        [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_DOWN] forKey:@"key_down"];
        [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_LEFT] forKey:@"key_left"];
        [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_RIGHT] forKey:@"key_right"];
        [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_FIRE] forKey:@"key_fire"];
        [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_CYCLE_WEAPONS] forKey:@"key_cycle_weapons"];
        [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_NOVA_BOMB] forKey:@"key_nova_bomb"];
        [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_PAUSE] forKey:@"key_pause"];


    }
    
    //[NSCursor hide];
    
    backgroundColour = [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0] retain];

    currentMenuItem = -1;
    
    frameCount = 0;
    sFrameRate = @"Unknown";
    bLocked = NO;

    paused = NO;
    
    if (timer)
    {
        [timer invalidate];
    }
    timer = [NSTimer	scheduledTimerWithTimeInterval:0
                                             target:self
                                           selector:@selector(runTimer)
                                           userInfo:self
                                            repeats:YES];

    // clear the keyboard map
    int loop;
    for(loop = 0; loop < 256; loop++)
    {
        keymap[loop] = NO;
    }

    for(loop = 0; loop < KEY_LAST; loop++)
    {
        keypressed[loop] = NO;
    }

    

    reprogramModeOn = NO;

    // clear all the tracking rectangles
    for(loop = 0; loop < BUTTONS_LAST; loop++)
    {
        buttons[loop] = -1;       
    }
            
    return self;
}

- (void)viewDidMoveToWindow
{
    [self setTrackingAreas];
    
    [super viewDidMoveToWindow];
}




- (void) dealloc
{
    if(backgroundColour)
    {
        [backgroundColour release];
    }

    [super dealloc];
}

- (BOOL)acceptsFirstResponder
{
    [[self window] makeFirstResponder:self];
    return YES;
}

- (void)drawRect:(NSRect)rect
{
    NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithCapacity:0];
    [attr setObject:[NSColor whiteColor] forKey:NSForegroundColorAttributeName];
        
    if(bLocked)
    {
        return;
    }

    bLocked = YES;

    //[self lockFocus];
    

    switch([game getGameStatus])
    {
        case GAME_PLAYING:
            [self drawGameDisplay];
            break;

        case GAME_OVER:
            [self drawGameOverDisplay];
            break;

        case GAME_TITLE:
            [self drawTitleDisplay];
            break;

        case GAME_LEVEL_OVER:
            [self drawLevelOverDisplay];
            break;

        case GAME_LIFE_OVER:
            [self drawLifeOverDisplay];
            break;

        case GAME_SHOP:
            [self drawShopDisplay];
            break;

        case GAME_DROID_SHOP:
            [self drawDroidShopDisplay];
            break;

        case GAME_PREFS:
            [self drawPrefsDisplay];
            break;

        case GAME_MAIN_MENU:
            [self drawMainMenuDisplay];
            break;

        case GAME_LEVEL_INTRO:
            [self drawLevelIntroDisplay];
            break;

        case GAME_HIGH_SCORES:
            [self drawHighScoresDisplay];
            break;

        case GAME_NEW_HIGH_SCORE:
            [self drawNewHighScoreDisplay];
            break;
    }

    /*
 
    // Calculate the current frame rate
    NSTimeInterval thisFrame = +[NSDate timeIntervalSinceReferenceDate];
    elapsed += (thisFrame - lastFrame);
    ++frameCount;
    lastFrame = thisFrame;

    if (elapsed > 1.0)
    {
        frameCount = 0;
        elapsed = 0.0;
    }
     */


    //[self unlockFocus];

    bLocked = NO;
    
}

- (void) setGame:(Game*) agame
{
    game = agame;
}

- (Game *) game
{
    return game;
}

- (void) setController:(Controller*) acontroller
{
    controller = acontroller;
}

- (void) runTimer
{
    int loop;
    int loopAlien;
    int loopPlant;
    int loopStrawberry;
    Tile* bullet;
    Tile* alien;
    Tile* explosion;
    Tile* powerUp;
    Tile* generator;
    Plant* plant;
    int deltaX;
    int deltaY;

    if(paused)
    {
        [self setNeedsDisplay:YES];

        return;
    }

    if([game getGameStatus] != GAME_PLAYING)
    {
        return;
    }
    
    [self processKeyboard];

    // Animate the player
    [[game getRobot] animate];

    /*
    // Animate the background tiles
    for(loop = 0; loop < [game getTileCount]; loop++)
    {
        //[[game getTileAt:loop] animate];
    }
     */

    // Animate the bullets
    for(loop = 0; loop < [game getBulletCount]; loop++)
    {
        [[game getBulletAt:loop] animate];
    }

    // Animate the aliens
    for(loop = 0; loop < [game getAlienCount]; loop++)
    {
        alien = [game getAlienAt:loop];

        // what type of alien is it?
        switch([alien getType])
        {
            case 0: // mindless crawlers
                break;

            case 1: // robot hunters
                if([[game getRobot] getPosition].y > [alien getPosition].y)
                {
                    deltaY = 3;
                }
                    else
                    {
                        deltaY = -3;
                    }

                    if([[game getRobot] getPosition].x > [alien getPosition].x)
                    {
                        deltaX = 3;
                    }
                    else
                    {
                        deltaX = -3;
                    }


                    [[alien getSprite] setMovement:deltaX:deltaY];
                break;
        }
        
        [alien animate];
    }

    // Animate the explosions
    for(loop = 0; loop < [game getExplosionCount]; loop++)
    {
        [[game getExplosionAt:loop] animate];
    }

    // Animate the power ups
    for(loop = 0; loop < [game getPowerUpCount]; loop++)
    {
        [[game getPowerUpAt:loop] animate];
    }

    // check for bullet hits
    bool hit = NO;
    
    for(loop = 0; loop < [game getBulletCount]; loop++)
    {
        bullet = [game getBulletAt:loop];
        hit = NO;
        
        if([bullet getLife] > 0)
        {
            for(loopAlien = 0; loopAlien < [game getAlienCount]; loopAlien++)
            {
                alien = [game getAlienAt:loopAlien];
                
                if([alien isWithinRect:[bullet getPosition].x:[bullet getPosition].y:30:30])
                {
                    [game createExplosion:[alien getPosition].x:[alien getPosition].y];
                    [game destroyAlienAt:loopAlien];
                    loopAlien--;

                    [game increaseKillCount];
                    [game changeScore:100];
                    hit = YES;
                }
            }

            if(hit)
            {
                [game destroyBulletAt:loop];
                loop--;
                
            }
        }
    }

    // check for alien hitting player
    for(loopAlien = 0; loopAlien < [game getAlienCount]; loopAlien++)
    {
        alien = [game getAlienAt:loopAlien];

        if([alien getLife] == -1)
        {
            if([alien isWithinRect:[[game getRobot] getPosition].x:[[game getRobot] getPosition].y:60:60])
            {
                // are the shields active?
                if([game getShields] > 0)
                {
                    [game createExplosion:[alien getPosition].x:[alien getPosition].y];
                    [game changeShields:-20];
                }
                else
                {
                    // End the level
                    [game setGameStatus:GAME_LIFE_OVER];
                    [self setTrackingAreas];
                }

                [game destroyAlienAt:loopAlien];

                [game increaseKillCount];
                
                break;
            }
        }
    }

    // check for alien hitting plant
    for(loopAlien = 0; loopAlien < [game getAlienCount]; loopAlien++)
    {
        alien = [game getAlienAt:loopAlien];

        for(loopPlant = 0; loopPlant < [game getPlantCount]; loopPlant++)
        {
            plant = [game getPlantAt:loopPlant];

            if([plant doesAlienEatPlant:[alien getPosition].x:[alien getPosition].y:[[alien getSprite] getWidth]:[[alien getSprite] getHeight]])
            {
                // Sound effect
                //[game playSound:2];
                
                [plant killPlant:1];
            }
        }

        // check for hitting strawberry
        for(loopStrawberry = 0; loopStrawberry < [game getPowerUpCount]; loopStrawberry++)
        {
            powerUp = [game getPowerUpAt:loopStrawberry];

            if([powerUp isWithinRect:[alien getPosition].x:[alien getPosition].y:[[alien getSprite] getWidth]:[[alien getSprite] getHeight]])
            {
                // has it hit a fruit?
                if([powerUp getType] == 3)
                {
                    // Sound effect
                    [game playSound:2];

                    [game destroyPowerUpAt:loopStrawberry];
                }
            }
        }
        
    }

    
   
     

    // check for player hitting power up
    for(loop = 0; loop < [game getPowerUpCount]; loop++)
    {
        powerUp = [game getPowerUpAt:loop];

        if([powerUp isWithinRect:[[game getRobot] getPosition].x:[[game getRobot] getPosition].y:60:60])
        {
            switch([powerUp getType])
            {
                case 1: // ammo
                    //[game changeScore:500];
                    [[game getWeaponAt:[game getWeapon]] changeAmmo:1];
                    break;

                case 2: // shields
                    [game changeShields:20];

                    // Sound effect
                    [game playSound:3];
                    break;

                case 3: // fruit

                    // ensure we have spare carrying capacity before picking up
                    if([game getFruitCarried] < [[game getRobot] getCarryCapacity])
                    {
                        [game changeScore:1000];
                        // Sound effect
                        [game playSound:3];

                        [powerUp setLife:0];
                        [game changeFruitCarried:1];
                    }
                    break;

                case 4: // bonus B
                    [game changeScore:500];
                    [game setBonusBall:BONUS_B
                                  Held:YES];

                    // Sound effect
                    [game playSound:3];

                    // check to see if all bonus balls held and trigger bonus
                    if([game allBonusBallsHeld])
                    {
                        [game setBonusLevel:NO];
                        [game clearAllObjects];
                        [game initLevel];

                        [game setGameStatus:GAME_LEVEL_INTRO];

                        [self setTrackingAreas];
                        [self setNeedsDisplay:YES];                        
                    }
                    
                    break;

                case 5: // bonus O
                    [game changeScore:500];
                    [game setBonusBall:BONUS_O
                                  Held:YES];

                    // Sound effect
                    [game playSound:3];

                    // check to see if all bonus balls held and trigger bonus
                    if([game allBonusBallsHeld])
                    {
                        [game setBonusLevel:NO];
                        [game clearAllObjects];
                        [game initLevel];

                        [game setGameStatus:GAME_LEVEL_INTRO];

                        [self setTrackingAreas];
                        [self setNeedsDisplay:YES];
                    }

                    break;

                case 6: // bonus N
                    [game changeScore:500];
                    [game setBonusBall:BONUS_N
                                  Held:YES];

                    // Sound effect
                    [game playSound:3];

                    // check to see if all bonus balls held and trigger bonus
                    if([game allBonusBallsHeld])
                    {
                        [game setBonusLevel:NO];
                        [game clearAllObjects];
                        [game initLevel];

                        [game setGameStatus:GAME_LEVEL_INTRO];

                        [self setTrackingAreas];
                        [self setNeedsDisplay:YES];
                    }
                    break;

                case 7: // bonus U
                    [game changeScore:500];
                    [game setBonusBall:BONUS_U
                                  Held:YES];

                    // Sound effect
                    [game playSound:3];

                    // check to see if all bonus balls held and trigger bonus
                    if([game allBonusBallsHeld])
                    {
                        [game setBonusLevel:NO];
                        [game clearAllObjects];
                        [game initLevel];

                        [game setGameStatus:GAME_LEVEL_INTRO];

                        [self setTrackingAreas];
                        [self setNeedsDisplay:YES];
                    }
                    break;

                case 8: // bonus S
                    [game changeScore:500];
                    [game setBonusBall:BONUS_S
                                  Held:YES];

                    // Sound effect
                    [game playSound:3];

                    // check to see if all bonus balls held and trigger bonus
                    if([game allBonusBallsHeld])
                    {
                        [game setBonusLevel:NO];
                        [game clearAllObjects];
                        [game initLevel];

                        [game setGameStatus:GAME_LEVEL_INTRO];

                        [self setTrackingAreas];
                        [self setNeedsDisplay:YES];
                    }

                    break;

                case 9: // fruit hoper
                    // check to see if we are carrying any fruit
                    if([game getFruitCarried] > 0)
                    {
                        [game changeScore:1000 * [game getFruitCarried]];
                        // Sound effect
                        [game playSound:3];

                        [game changeFruitCollected:[game getFruitCarried]];
                        [game setFruitCarried:0];
                    }
                    break;

                case 10: // points bonus
                    [game changeScore:1000];
                    [powerUp setLife:0];

                    // Sound effect
                    [game playSound:3];
                    
                    break;
            }

            if([powerUp getLife] != -1)
            {
                [game destroyPowerUpAt:loop];
            }
            break;
        }
    }


    // handle weapon delay counters
    // Draw the head up display
    for(loop = 0; loop < [game getWeaponCount]; loop++)
    {
        [[game getWeaponAt:loop] decreaseDelayCounter];
    }
    
    // clean up dead bullets
    for(loop = 0; loop < [game getBulletCount]; loop++)
    {
        bullet = [game getBulletAt:loop];

        if([bullet getLife] == 0)
        {
            [game destroyBulletAt:loop];
            loop--;
        }
    }

    // clean up dead explosions
    for(loop = 0; loop < [game getExplosionCount]; loop++)
    {
        explosion = [game getExplosionAt:loop];

        if([explosion getLife] == 0)
        {
            [game destroyExplosionAt:loop];
            loop--;
        }
    }

    // clean up dead power ups
    for(loop = 0; loop < [game getPowerUpCount]; loop++)
    {
        powerUp = [game getPowerUpAt:loop];

        if([powerUp getLife] == 0)
        {
            [game destroyPowerUpAt:loop];
            loop--;
        }
    }

   
    // spawn another alien if we have not reached the limit
    if([game decreaseSpawnCount]
       && [game getGeneratorCount] > 0)
    {
        // maxium pests of 10
        if([game getAlienCount] < 10)
        {
            [game setSpawnCount:[game getSpawnRate]];

            // spawn rate does not increase per level
            //for(loop = 0; loop < [game getLevel]; loop++)
            {
                int useGenerator = rand()%([game getGeneratorCount]);

                generator = [game getGeneratorAt:useGenerator];

                int deltaX = (rand()%10) - 5;
                int deltaY = (rand()%10) - 5;

                // what type of alien shall we create?
                // 0 - mindless crawlers
                // 1 - robot hunters
                
                int type = (rand()%2);

                // prevent Aliens that only bounce up and down
                if(deltaX == 0)
                {
                    deltaX = 5;
                }

                [game createAlien:[generator getPosition].x:[generator getPosition].y :deltaX:deltaY:type];
            }
           

            // add a power up?
            if(rand()%10 < 2)
            {
                // which type of power up to add?
                switch(rand()%7)
                {
                    case 0:
                        // shield power up
                        [game createPowerUp:(rand()%600) + 50:(rand()%400) + 50:(rand()%10)-5:(rand()%10)-5:2:100];
                        break;

                    case 1:
                        // Give me a 'B'
                        [game createPowerUp:(rand()%600) + 50:(rand()%400) + 50:(rand()%10)-5:(rand()%10)-5:4:100];
                        break;

                    case 2:
                        // Give me a 'O'
                        [game createPowerUp:(rand()%600) + 50:(rand()%400) + 50:(rand()%10)-5:(rand()%10)-5:5:100];
                        break;

                    case 3:
                        // Give me a 'N'
                        [game createPowerUp:(rand()%600) + 50:(rand()%400) + 50:(rand()%10)-5:(rand()%10)-5:6:100];
                        break;

                    case 4:
                        // Give me a 'U'
                        [game createPowerUp:(rand()%600) + 50:(rand()%400) + 50:(rand()%10)-5:(rand()%10)-5:7:100];
                        break;

                    case 5:
                        // Give me a 'S'
                        [game createPowerUp:(rand()%600) + 50:(rand()%400) + 50:(rand()%10)-5:(rand()%10)-5:8:100];
                        break;

                    case 6:
                        // points bonus
                        [game createPowerUp:(rand()%600) + 50:(rand()%400) + 50:(rand()%10)-5:(rand()%10)-5:10:100];
                        break;
                }
            }

            // grow plants
            for(loop = 0; loop < [game getPlantCount]; loop++)
            {
                plant = [game getPlantAt:loop];
                
                [plant growPlant:1];

                // create fruit if at height milestone
                if([plant getHeight] % 3 == 0)
                {
                    [game createPowerUp:[plant getPositionX] + /*(rand()%60) - 30*/ - 15:[plant getPositionY] + ([plant getHeight] * 10) - 30:0:0:3:-1];
                }
                
            }

            // check to see if plants are fully grown
            for(loop = 0; loop < [game getPlantCount]; loop++)
            {
                plant = [game getPlantAt:loop];
            }
        }
    }

    // decrease remaining time
    [game changeTimeRemaining:-1];

    // check for end of level
    if([game getTimeRemaining] == 0)
    {
        // end the level
        [game setGameStatus:GAME_LEVEL_OVER];
        [self setTrackingAreas];

        // give cash bonus for completing the level
        [game changeMoney:([game getKillCount] * 10) + ([game getFruitCollected] * 500)];
    }
    
    [self setNeedsDisplay:YES];
}

- (void)keyUp:(NSEvent *)theEvent
{
    int key = (int)[[theEvent characters] characterAtIndex:0];

    int loop;

    for(loop = 0; loop < KEY_LAST; loop++)
    {
        if(key == keys[loop])
        {
            keypressed[loop] = NO;
        }
    }

    //NSLog(@"%d", key);

    // are we entering a new highscore name?
    if([game getGameStatus] == GAME_NEW_HIGH_SCORE)
    {
        NSMutableArray	*gameScores, *gameNames, *gameLevels;
        NSArray* hiScores;

        hiScores = [controller getHiScores];

        // reset arrays with gameLevel
        gameNames = [hiScores objectAtIndex:0];
        gameScores = [hiScores objectAtIndex:1];
        gameLevels = [hiScores objectAtIndex:2];

        if(key == 127) // delete
        {
            // ensure we have more than 0 characters
            if([[gameNames objectAtIndex:currentHighScoreRanking] cStringLength] > 0)
            {
                [gameNames replaceObjectAtIndex:currentHighScoreRanking
                                     withObject:[[gameNames objectAtIndex:currentHighScoreRanking] substringToIndex:[[gameNames objectAtIndex:currentHighScoreRanking] cStringLength] - 1]];
            }
        }
        else if(key != 13) // ensure return not being pressed
        {
            // ensure we have no more than 20 characters
            if([[gameNames objectAtIndex:currentHighScoreRanking] cStringLength] < 20)
            {
                [gameNames replaceObjectAtIndex:currentHighScoreRanking
                                     withObject:[[gameNames objectAtIndex:currentHighScoreRanking] stringByAppendingFormat:@"%c",
                                         key]];
            }
        }
        
        [[NSUserDefaults standardUserDefaults] setObject:hiScores
                                                  forKey:@"hiScores"];
        
        [self setNeedsDisplay:YES];
    }

    // are we reprogramming the keys?
    if([game getGameStatus] == GAME_PREFS)
    {
        if(reprogramModeOn == YES)
        {
            keys[currentReprogramKey] = key;

            currentReprogramKey++;

            if(currentReprogramKey == KEY_LAST)
            {
                // save the keyboard settings
                [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_UP] forKey:@"key_up"];
                [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_DOWN] forKey:@"key_down"];
                [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_LEFT] forKey:@"key_left"];
                [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_RIGHT] forKey:@"key_right"];
                [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_FIRE] forKey:@"key_fire"];
                [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_CYCLE_WEAPONS] forKey:@"key_cycle_weapons"];
                [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_NOVA_BOMB] forKey:@"key_nova_bomb"];
                [[NSUserDefaults standardUserDefaults] setInteger:keys[KEY_PAUSE] forKey:@"key_pause"];
                
                reprogramModeOn = NO;
            }

            [self setNeedsDisplay:YES];
        }
        
        if(key == (int)'k' && reprogramModeOn == NO)
        {
            reprogramModeOn = YES;

            currentReprogramKey = KEY_LEFT;
        }

    }

    switch(key)
    {
        case (int) 'd':
            if([game getGameStatus] == GAME_MAIN_MENU)
            {
                [game setGameStatus:GAME_DROID_SHOP];

                [self setNeedsDisplay:YES];
            }
            break;

        case (int) 's':
            if([game getGameStatus] == GAME_MAIN_MENU
               && [game getCurrentRobot] != -1)
            {
                [game setGameStatus:GAME_SHOP];

                [self setNeedsDisplay:YES];
            }
            break;

        case NSF12FunctionKey: // cash boost
            if([game getGameStatus] == GAME_MAIN_MENU)
            {
                [game changeMoney:10000];
                [self setNeedsDisplay:YES];
            }
            break;

        case (int) '0':

            switch([game getGameStatus])
            {
               
                                   

                case GAME_MAIN_MENU:

                    // ensure we have a droid to go harvesting with
                    if([game getCurrentRobot] != -1)
                    {
                        [game clearAllObjects];
                        [game initLevel];

                        [game setGameStatus:GAME_PLAYING];
                        [self setNeedsDisplay:YES];
                    }
                    break;

                    
            }

            [self setTrackingAreas];


        break;
    }
}


- (void)keyDown:(NSEvent *)theEvent
{
    
    int key = (int)[[theEvent characters] characterAtIndex:0];

    //NSLog(@"key %d [%c]", key, key);
    int loop;

    for(loop = 0; loop < KEY_LAST; loop++)
    {
        if(key == keys[loop])
        {
            keypressed[loop] = YES;
        }
    }

    if(reprogramModeOn)
    {
        return;
    }

    if([game getGameStatus] == GAME_PLAYING)
    {
        if(keypressed[KEY_CYCLE_WEAPONS])
        {
            [game cycleWeapons];
        }

        if(keypressed[KEY_NOVA_BOMB])
        {
            // check we have a stock of Nova Bombs
            if([game getNovaBombCount] > 0)
            {
                [game useNovaBomb];
            }
        }

        if(keypressed[KEY_PAUSE])
        {
            if(!paused)
            {
                paused = YES;
                [timer invalidate];
            }
            else
            {
                paused = NO;
                timer = [NSTimer	scheduledTimerWithTimeInterval:0
                                                         target:self
                                                       selector:@selector(runTimer)
                                                       userInfo:self
                                                        repeats:YES];

            }            
        }
    }
    
     
    switch(key)
    {
        case 27: // escape
            
            if([game getGameStatus] == GAME_PLAYING)
            {
                [timer invalidate];
            }

            [game setGameStatus:GAME_TITLE];

            [self setTrackingAreas];

            [self setNeedsDisplay:YES];
            break;
        
    }

    
}

- (void)drawPowerBar:(int)x:(int)y:(float)length:(int)height:(float)value:(float)maxValue:(float)red:(float)green:(float)blue
{
    float scaledMax;	// max value scaled to bar length
    float scaledValue;	// current value scaled to bar length

    scaledMax = length / maxValue;
    scaledValue = value * scaledMax;

    NSRect rcBar = NSMakeRect(x, y, scaledValue, height);
    [[NSColor colorWithDeviceRed:red green:green blue:blue alpha:1.0] set];
    NSRectFill(rcBar);
    rcBar = NSMakeRect(x + scaledValue, y, length - scaledValue, height);
    [[NSColor colorWithDeviceRed:red green:green blue:blue alpha:0.2] set];
    NSRectFill(rcBar);
    
}

- (void)processKeyboard
{
    int deltaX;
    int deltaY;

    if([game areStabilisersInstalled])
    {
        deltaX = 0;
        deltaY = 0;
    }
    else
    {
        deltaX = [[[game getRobot] getSprite] getMovementX];
        deltaY = [[[game getRobot] getSprite] getMovementY];
    }
    
    if(keypressed[KEY_LEFT])
    {

        deltaX = -[[game getRobot] getSpeed];
        [[game getRobot] setFireDirection:-5:5];
    }

    if(keypressed[KEY_RIGHT])
    {

        deltaX = [[game getRobot] getSpeed];
        [[game getRobot] setFireDirection:5:5];
    }

    if(keypressed[KEY_UP])
    {

        if([[[game getRobot] getSprite] isGravityOn])
        {
            deltaY = 10;
        }
        else
        {
            deltaY = [[game getRobot] getSpeed];
        }
    }

    if(keypressed[KEY_DOWN])
    {
        deltaY = -[[game getRobot] getSpeed];
    }
        
    if(keypressed[KEY_FIRE])
    {
        if([[game getWeaponAt:[game getWeapon]] getAmmo] > 0)
        {
            Weapon* weapon = [game getWeaponAt:[game getWeapon]];

            if([weapon canFire])
            {
                // Sound effect
                [game playSound:4];
                
                int type;

                if([weapon doesItBounce])
                {
                    type = 1;
                }
                else
                {
                    type = 2;
                }

                if([weapon firesForward])
                {
                    [game createBullet:[[game getRobot] getFireX] * 2:0:type];

                    [weapon changeAmmo:-1];
                }

                if([weapon firesBackward])
                {
                    [game createBullet:[[game getRobot] getFireX] * -2:0:type];

                    [weapon changeAmmo:-1];
                }

                if([weapon firesUp])
                {
                    [game createBullet:0:[[game getRobot] getFireY] * 2:type];

                    [weapon changeAmmo:-1];
                }

                if([weapon firesDown])
                {
                    [game createBullet:0:[[game getRobot] getFireY] * -2:type];

                    [weapon changeAmmo:-1];
                }

                // set the counter before we can fire again
                [weapon resetDelayCounter];
            }
        }
    }


    // set player movement based on keypresses
    [[[game getRobot] getSprite] setMovement:deltaX:deltaY];
}



- (void)drawTitleDisplay
{
    NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithCapacity:0];
    [attr setObject:[NSColor whiteColor] forKey:NSForegroundColorAttributeName];

    [backgroundColour set];
    //NSRectFill([self bounds]);
    //[[NSBezierPath bezierPathWithRect:[self bounds]] fill];

    [[NSImage imageNamed:@"title screen"] compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];

    
    [self displayText:@"Garden Pests Version 1.0"
                  AtX:625
                  AtY:115
             FontSize:12];
    
    [self displayText:@"By Andrew Sage"
                  AtX:625
                  AtY:100
             FontSize:12];

    /*
    [self displayText:@"(Retro Engine Version 0.1)"
                  AtX:625
                  AtY:70
             FontSize:12];
     */
    
    [self displayText:@"Email: pests@cloisterroom.com"
                  AtX:625
                  AtY:40
             FontSize:10];
    
    [self displayText:@"AIM: cloisterroom@mac.com"
                  AtX:625
                  AtY:25
             FontSize:10];
    
    [self displayText:@"Web: www.cloisterroom.com"
                  AtX:625
                  AtY:10
             FontSize:10];

    
    [self displayText:@"uDevGame 2003 Entry - Visit iDevGames.com for more games!"
                  AtX:200
                  AtY:580
             FontSize:14];

    [self displayText:@"Use your Garden Droid to harvest the Galactic Strawberries as you battle to protect the plants from the garden pests.":5:130:600:40];   
    
    [self displayText:@"'Escape' - quit game"
                  AtX:5
                  AtY:80
             FontSize:12];
    
    // Display help information
    [self displayText:@"Garden Droid"
                  AtX:10
                  AtY:10
             FontSize:10];
    
    [[NSImage imageNamed:@"rendered robot1"] compositeToPoint:NSMakePoint(15, 20) operation:NSCompositeSourceOver];

    [self displayText:@"Garden Pest"
                  AtX:110
                  AtY:10
             FontSize:10];
    
    [[NSImage imageNamed:@"crawler2"] compositeToPoint:NSMakePoint(110, 20) operation:NSCompositeSourceOver];

    [self displayText:@"Ammo Dump"
                  AtX:210
                  AtY:10
             FontSize:10];
    
    [[NSImage imageNamed:@"refuel1"] compositeToPoint:NSMakePoint(215, 20) operation:NSCompositeSourceOver];

    [self displayText:@"Shields"
                  AtX:320
                  AtY:10
             FontSize:10];
    
    [[NSImage imageNamed:@"robot1"] compositeToPoint:NSMakePoint(310, 20) operation:NSCompositeSourceOver];

    [self displayText:@"Pest Generator"
                  AtX:400
                  AtY:10
             FontSize:10];
    
    [[NSImage imageNamed:@"generator1"] compositeToPoint:NSMakePoint(410, 20) operation:NSCompositeSourceOver];

    [self displayText:@"Galactic Strawberry"
                  AtX:510
                  AtY:10
             FontSize:10];
    
    [[NSImage imageNamed:@"fruit1"] compositeToPoint:NSMakePoint(530, 25) operation:NSCompositeSourceOver];

    

    // Draw bar between instructions and splash screen
    NSRect rcBar = NSMakeRect(0, 135, 800, 2);
    NSColor *fillColour;

    fillColour = [[NSColor colorWithDeviceRed:1.0 green:1.0 blue:1.0 alpha:1.0] colorWithAlphaComponent:0.5];

    [fillColour set];

    NSRectFillUsingOperation(rcBar, NSCompositeSourceOver);

    rcBar = NSMakeRect(620, 0, 2, 135);
    NSRectFillUsingOperation(rcBar, NSCompositeSourceOver);

    // Draw the buttons
    [[NSImage imageNamed:@"button_options"] compositeToPoint:NSMakePoint(500, 350) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_OPTIONS ? 1.0 : 0.5];

    [[NSImage imageNamed:@"button_highscores"]  compositeToPoint:NSMakePoint(500, 280) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_HIGH_SCORES ? 1.0 : 0.5];

    [[NSImage imageNamed:@"button_play"] compositeToPoint:NSMakePoint(500, 210) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_PLAY ? 1.0 : 0.5];

    

    // Display uDevGame 2003 logo
    [[NSImage imageNamed:@"uDevGame2003"] compositeToPoint:NSMakePoint(700, 480) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_UDEVGAME ? 1.0 : 0.5];

    [self displayText:@"Click logo to vote"
                  AtX:700
                  AtY:460
             FontSize:11];

}

- (void)drawGameDisplay
{
    NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithCapacity:0];
    [attr setObject:[NSColor whiteColor] forKey:NSForegroundColorAttributeName];

    NSString* sStatus;

    int loop;
    
    //[backgroundColour set];
    //NSRectFill([self bounds]);
    //[[NSBezierPath bezierPathWithRect:[self bounds]] fill];

    // Draw the background
    [[game getBackgroundImage] compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];

    // Draw the tiles
    for(loop = 0; loop < [game getTileCount]; loop++)
    {
        [[game getTileAt:loop] drawTile:1:TILE_WIDTH/2:TILE_HEIGHT/2];
    }

    // Draw the generators
    for(loop = 0; loop < [game getGeneratorCount]; loop++)
    {
        [[game getGeneratorAt:loop] drawTile:1:TILE_WIDTH/2:TILE_HEIGHT/2];
    }

    // Draw the plants
    for(loop = 0; loop < [game getPlantCount]; loop++)
    {
        [[game getPlantAt:loop] drawPlant];
    }
    
    // Draw the aliens
    for(loop = 0; loop < [game getAlienCount]; loop++)
    {
        [[game getAlienAt:loop] drawTile:1:TILE_WIDTH/2:TILE_HEIGHT/2];
    }

    // Draw the bullets
    for(loop = 0; loop < [game getBulletCount]; loop++)
    {
        [[game getBulletAt:loop] drawTile:1:TILE_WIDTH/2:TILE_HEIGHT/2];
    }

    // Draw the explosions
    for(loop = 0; loop < [game getExplosionCount]; loop++)
    {
        [[game getExplosionAt:loop] drawTile:1:TILE_WIDTH/2:TILE_HEIGHT/2];
    }

    // Draw the power ups
    for(loop = 0; loop < [game getPowerUpCount]; loop++)
    {
        [[game getPowerUpAt:loop] drawTile:1:TILE_WIDTH/2:TILE_HEIGHT/2];
    }

    // Draw the player
    [[game getRobot] drawTile:1:TILE_WIDTH/2:TILE_HEIGHT/2];

    // Display current game information
    NSColor *fillColour;

    fillColour = [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.5 alpha:1.0] colorWithAlphaComponent:1.0];

    [fillColour set];


    /*
    // Display the frame rate information
    NSPoint ptFrameRate = NSMakePoint(10, 10);
    if(elapsed > 0.0)
    {
        sFrameRate = [NSString stringWithFormat:@"%3.4f FPS",
            frameCount / elapsed];
        
        [sFrameRate drawAtPoint:ptFrameRate withAttributes:attr];
    }
     */
     

    // lower bar
    NSRectFillUsingOperation(NSMakeRect(45, 560, 550, 15), NSCompositeSourceOver);

    // lower display
    sStatus = [NSString stringWithFormat:@"High Score: %06d  Score: %06d  Credits: %06d  Level: %02d  Nova Bombs: %d",
        [game getHighScore],
        [game getScore],
        [game getMoney],
        [game getLevel],
        [game getNovaBombCount]
        ];

    [self displayText:sStatus
                  AtX:50
                  AtY:560
             FontSize:12];

    // display weapon
    [self drawPowerBar:655:560:100:15:[[game getWeaponAt:[game getWeapon]] getAmmo]:[[game getWeaponAt:[game getWeapon]] getMaxAmmo]:0.0:0.5:0.0];
    [self displayText:[[game getWeaponAt:[game getWeapon]] getTitle]
                  AtX:660
                  AtY:560
             FontSize:12];

    // display shields level
    [self drawPowerBar:655:580:100:15:[game getShields]:100:1.0:0.0:0.0];
    [self displayText:@"Shields"
                  AtX:685
                  AtY:580
             FontSize:12];

    // display strawberries carried
    [self drawPowerBar:655:540:100:15:[game getFruitCarried]:[[game getRobot] getCarryCapacity]:0.7:0.0:0.0];
    [self displayText:@"Carried"
                  AtX:660
                  AtY:540
             FontSize:12];


    // display time remaining
    [self drawPowerBar:45:580:550:15:[game getTimeRemaining]/10:2500/2/10:0.5:0.0:0.5];
    [self displayText:@"Time Remaining"
                  AtX:270
                  AtY:580
             FontSize:12];

    // display fruit counter
    [[NSImage imageNamed:@"fruit1"] compositeToPoint:NSMakePoint(600, 545) operation:NSCompositeSourceOver];
    [self displayText:[NSString stringWithFormat:@"%03d", [game getFruitCollected]]
                  AtX:618
                  AtY:560
             FontSize:12];

    // Draw the bonus balls
    if([game isBonusBallHeld:BONUS_B])
    {
        [[NSImage imageNamed:@"bonus_B1"] compositeToPoint:NSMakePoint(120, 0) operation:NSCompositeSourceOver];
    }

    if([game isBonusBallHeld:BONUS_O])
    {
        [[NSImage imageNamed:@"bonus_O1"] compositeToPoint:NSMakePoint(240, 0) operation:NSCompositeSourceOver];
    }

    if([game isBonusBallHeld:BONUS_N])
    {
        [[NSImage imageNamed:@"bonus_N1"] compositeToPoint:NSMakePoint(360, 0) operation:NSCompositeSourceOver];
    }

    if([game isBonusBallHeld:BONUS_U])
    {
        [[NSImage imageNamed:@"bonus_U1"] compositeToPoint:NSMakePoint(480, 0) operation:NSCompositeSourceOver];
    }

    if([game isBonusBallHeld:BONUS_S])
    {
        [[NSImage imageNamed:@"bonus_S1"] compositeToPoint:NSMakePoint(600, 0) operation:NSCompositeSourceOver];
    }

    // are we in pause mode?
    if(paused)
    {
        [self displayText:@"P  A  U  S  E  D"
                      AtX:300
                      AtY:300
                 FontSize:30];
    }

}

- (void)drawGameOverDisplay
{
    [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0] set];
    NSRectFill([self bounds]);

    [self displayText:@"No money left for a new Garden Droid, so..."
                  AtX:200
                  AtY:560
             FontSize:20];

    [[NSImage imageNamed:@"game over"] compositeToPoint:NSMakePoint(200, 100) operation:NSCompositeSourceOver];

    // Draw the buttons
    [[NSImage imageNamed:@"button_continue"] compositeToPoint:NSMakePoint(262, 20) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_CONTINUE ? 1.0 : 0.5];


}

- (void)drawMainMenuDisplay
{
    double fade;	// alpha level for menu options
    
    [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0] set];
    NSRectFill([self bounds]);

    [[[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.5 alpha:1.0] colorWithAlphaComponent:0.5] set];

    // display current cash flow
    [self displayText:@"Credits"
                  AtX:10
                  AtY:220
             FontSize:15];

    [self displayText:[NSString stringWithFormat:@"%06d",[game getMoney]]
                  AtX:10
                  AtY:200
             FontSize:20];
    

    [self displayText:@"Garden Droid"
                  AtX:10
                  AtY:180
             FontSize:15];
    

    
    // display the current robot selected
    if([game getCurrentRobot] != -1)
    {
        [self displayText:[[game getRobotAt:[game getCurrentRobotItem]] getTitle]
                      AtX:10
                      AtY:160
                 FontSize:20];
        
        // Only allow harvesting if a Garden Droid is owned
        fade = 1.0;
    }
    else
    {
        [self displayText:@"No Droid Owned"
                      AtX:10
                      AtY:160
                 FontSize:20];

        fade = 0.5;
    }


    // display the Growing Domes image
    [[NSImage imageNamed:@"domes small"] compositeToPoint:NSMakePoint(242, 100) operation:NSCompositeSourceOver fraction:currentMenuItem == MENU_DOMES ? fade : 0.5];

    
    // display the Upgrades image
    [[NSImage imageNamed:@"upgrade store small"] compositeToPoint:NSMakePoint(480, 350) operation:NSCompositeSourceOver fraction:currentMenuItem == MENU_UPGRADES ? fade : 0.5];

    

    // display the Droid store image
    [[NSImage imageNamed:@"droid store small"] compositeToPoint:NSMakePoint(5, 350) operation:NSCompositeSourceOver fraction: currentMenuItem == MENU_DROID_SHOP ? 1.0 : 0.5];

    switch(currentMenuItem)
    {
        case MENU_DOMES:
            if([game getCurrentRobot] != -1)
            {
                [self displayText:@"Enter the Growing Domes to go Galactic Strawberry harvesting.": 242:100:315:100];
            }
            else
            {
                [self displayText:@"You must first purchase a Garden Droid before you can go Galactic Strawberry harvesting.": 242:100:315:100];
            }
            
            break;

        case MENU_UPGRADES:
            if([game getCurrentRobot] != -1)
            {
                [self displayText:@"Purchase weapons and other additional hardware from Upgrades 4 U.":480:350:315:100];
            }
            else
            {
                [self displayText:@"You must first purchase a Garden Droid before you can buy any upgrades.":480:350:315:100];
            }
            break;

        case MENU_DROID_SHOP:
            [self displayText:@"Visit The Droid Emporium to purchase a Garden Droid for harvesting Galactic Strawberries.":5:350:315:100];
            break;
    }
}

- (void)drawLifeOverDisplay
{
    [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0] set];
    NSRectFill([self bounds]);

    [[NSImage imageNamed:@"droid destroyed"] compositeToPoint:NSMakePoint(200, 200) operation:NSCompositeSourceOver];

    if([game isInsuranceOwned])
    {
        [self displayText:@"Luckily you had that insurance policy!"
                      AtX:220
                      AtY:160
                 FontSize:20];
    }
        
    // Draw the buttons
    [[NSImage imageNamed:@"button_continue"] compositeToPoint:NSMakePoint(262, 20) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_CONTINUE ? 1.0 : 0.5];

}

- (void)drawLevelOverDisplay
{
    [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0] set];

    NSRectFill([self bounds]);

    [[NSImage imageNamed:@"level over"] compositeToPoint:NSMakePoint(200, 200) operation:NSCompositeSourceOver];
  
    // display earnings information
    [self displayText:[NSString stringWithFormat:@"Kill Bonus %d x 10",
        [game getKillCount]]
                  AtX:300
                  AtY:220
             FontSize:15];
    
    [self displayText:[NSString stringWithFormat:@"%06d",
        [game getKillCount] * 10]
                  AtX:300
                  AtY:200
             FontSize:20];

    [self displayText:[NSString stringWithFormat:@"Fruit Bonus %d x 500",
        [game getFruitCollected]]
                  AtX:300
                  AtY:180
             FontSize:15];

    [self displayText:[NSString stringWithFormat:@"%06d",
         [game getFruitCollected] * 500]
                  AtX:300
                  AtY:160
             FontSize:20];

    [self displayText:[NSString stringWithFormat:@"Environmental Fine %d x %d x 10",
        [game getNovaBombUsedCount],
        [game getFruitCollected]]
                  AtX:300
                  AtY:140
             FontSize:15];

    [self displayText:[NSString stringWithFormat:@"-%06d",
        [game getNovaBombUsedCount] * [game getFruitCollected] * 50]
                  AtX:300
                  AtY:120
             FontSize:20];
    

    [self displayText:@"Total"
                  AtX:300
                  AtY:100
             FontSize:15];

    [self displayText:[NSString stringWithFormat:@"%06d credits",
        ([game getKillCount] * 10) + ([game getFruitCollected] * 500)
        - ([game getNovaBombUsedCount] * [game getFruitCollected] * 50)]
                  AtX:300
                  AtY:80
             FontSize:20];

    // Draw the buttons
    [[NSImage imageNamed:@"button_continue"] compositeToPoint:NSMakePoint(262, 20) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_CONTINUE ? 1.0 : 0.5];
}

- (void)drawLevelIntroDisplay
{
    [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0] set];

    NSRectFill([self bounds]);

    [self displayText:@"Have you visited iDevGames to vote yet?"
                  AtX:250
                  AtY:550
             FontSize:15];

    if([game isBonusLevel])
    {
        [self displayText:@"Bonus Level"
                      AtX:320
                      AtY:400
                 FontSize:30];

        [self displayText:@"Collect the bonus points..."
                      AtX:280
                      AtY:300
                 FontSize:20];

        [[NSImage imageNamed:@"bonus2"] compositeToPoint:NSMakePoint(370, 150) operation:NSCompositeSourceOver];
        
    }
    else
    {
        [self displayText:[NSString stringWithFormat:@"Level %03d", [game getLevel]]
                      AtX:320
                      AtY:500
                 FontSize:30];

        [self displayText:@"Collect Galactic Strawberries..."
                      AtX:250
                      AtY:410
                 FontSize:20];

        [[NSImage imageNamed:@"fruit1"] compositeToPoint:NSMakePoint(370, 350) operation:NSCompositeSourceOver];

        [self displayText:@"...and drop them off in the Hopper..."
                      AtX:250
                      AtY:300
                 FontSize:20];

        [[NSImage imageNamed:@"hopper1"] compositeToPoint:NSMakePoint(370, 240) operation:NSCompositeSourceOver];

        [self displayText:@"...while avoiding the Garden Pests"
                      AtX:250
                      AtY:190
                 FontSize:20];

        [[NSImage imageNamed:@"crawler2"] compositeToPoint:NSMakePoint(310, 150) operation:NSCompositeSourceOver];

        [[NSImage imageNamed:@"hunter2"] compositeToPoint:NSMakePoint(430, 125) operation:NSCompositeSourceOver];
    }
        
    // Draw the buttons
    [[NSImage imageNamed:@"button_continue"] compositeToPoint:NSMakePoint(262, 20) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_CONTINUE ? 1.0 : 0.5];
}



- (void)drawPrefsDisplay
{
    [backgroundColour set];
    NSRectFill([self bounds]);

    [[NSImage imageNamed:@"button_options"] compositeToPoint:NSMakePoint(262, 500) operation:NSCompositeSourceOver];
    

    [self displayText:@"Left"
                  AtX:100
                  AtY:300
             FontSize:12];
    
    [self displayText:@"Right"
                  AtX:100
                  AtY:280
             FontSize:12];
    
    [self displayText:@"Up"
                  AtX:100
                  AtY:260
             FontSize:12];
    
    [self displayText:@"Down"
                  AtX:100
                  AtY:240
             FontSize:12];
    
    [self displayText:@"Fire"
                  AtX:100
                  AtY:220
             FontSize:12];
    
    [self displayText:@"Cycle Weapons"
                  AtX:100
                  AtY:200
             FontSize:12];
    
    [self displayText:@"Nova Bomb"
                  AtX:100
                  AtY:180
             FontSize:12];
    
    [self displayText:@"Pause"
                  AtX:100
                  AtY:160
             FontSize:12];
    
    [self displayText:[self displayKey:keys[KEY_LEFT]]
                  AtX:200
                  AtY:300
             FontSize:12];
    
    [self displayText:[self displayKey:keys[KEY_RIGHT]]
                  AtX:200
                  AtY:280
             FontSize:12];
    
    [self displayText:[self displayKey:keys[KEY_UP]]
                  AtX:200
                  AtY:260
             FontSize:12];
    
    [self displayText:[self displayKey:keys[KEY_DOWN]]
                  AtX:200
                  AtY:240
             FontSize:12];
    
    [self displayText:[self displayKey:keys[KEY_FIRE]]
                  AtX:200
                  AtY:220
             FontSize:12];
    
    [self displayText:[self displayKey:keys[KEY_CYCLE_WEAPONS]]
                  AtX:200
                  AtY:200
             FontSize:12];
    
    [self displayText:[self displayKey:keys[KEY_NOVA_BOMB]]
                  AtX:200
                  AtY:180
             FontSize:12];
    
    [self displayText:[self displayKey:keys[KEY_PAUSE]]
                  AtX:200
                  AtY:160
             FontSize:12];


    [self displayText:@"Press 'K' to customise keys"
                  AtX:100
                  AtY:400
             FontSize:12];

    // Draw the buttons
    [[NSImage imageNamed:@"button_done"] compositeToPoint:NSMakePoint(262, 10) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_DONE ? 1.0 : 0.5];

    if([game isMusicOn])
    {
        [[NSImage imageNamed:@"button_music_off"] compositeToPoint:NSMakePoint(450, 350) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_MUSIC ? 1.0 : 0.5];

    }
    else
    {
        [[NSImage imageNamed:@"button_music_on"] compositeToPoint:NSMakePoint(450, 350) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_MUSIC ? 1.0 : 0.5];
    }

    if([game isSoundOn])
    {
        [[NSImage imageNamed:@"button_sound_off"] compositeToPoint:NSMakePoint(450, 300) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_SOUND ? 1.0 : 0.5];
    }
    else
    {
        [[NSImage imageNamed:@"button_sound_on"] compositeToPoint:NSMakePoint(450, 300) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_SOUND ? 1.0 : 0.5];
    }
    
}

- (void)drawHighScoresDisplay
{
    NSMutableArray	*gameScores, *gameNames, *gameLevels;
    NSArray* hiScores;
    int loop;
    
    [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0] set];

    //NSRectFill([self bounds]);

    [[NSImage imageNamed:@"backdrop1"] compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];


    [[NSImage imageNamed:@"button_highscores"] compositeToPoint:NSMakePoint(262, 500) operation:NSCompositeSourceOver];


    hiScores = [controller getHiScores];


    // reset arrays with gameLevel
    gameNames = [hiScores objectAtIndex:0];
    gameScores = [hiScores objectAtIndex:1];
    gameLevels = [hiScores objectAtIndex:2];

    [self displayText:@"Score"
                  AtX:230
                  AtY:400
             FontSize:20];

    [self displayText:@"Level"
                  AtX:350
                  AtY:400
             FontSize:20];

    [self displayText:@"Name"
                  AtX:450
                  AtY:400
             FontSize:20];
    
    for (loop = 0; loop < 10; loop++)
    {
        [self displayText:[NSString stringWithFormat:@"%08d", [[gameScores objectAtIndex:loop] intValue]]
                      AtX:230
                      AtY:400 - (loop * 25) - 25
                 FontSize:20];

        [self displayText:[NSString stringWithFormat:@"%03d", [[gameLevels objectAtIndex:loop] intValue]]
                      AtX:350
                      AtY:400 - (loop * 25) - 25
                 FontSize:20];

        [self displayText:[gameNames objectAtIndex:loop]
                      AtX:450
                      AtY:400 - (loop * 25) - 25
                 FontSize:20];
    }
    
    // Draw the buttons
    [[NSImage imageNamed:@"button_continue"] compositeToPoint:NSMakePoint(262, 20) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_CONTINUE ? 1.0 : 0.5];
}

- (void)drawNewHighScoreDisplay
{
    NSMutableArray	*gameScores, *gameNames, *gameLevels;
    NSArray* hiScores;
    int loop;

    [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0] set];

    NSRectFill([self bounds]);


    [self displayText:@"Well done! Enter your name."
                  AtX:300
                  AtY:560
             FontSize:20];

    hiScores = [controller getHiScores];


    // reset arrays with gameLevel
    gameNames = [hiScores objectAtIndex:0];
    gameScores = [hiScores objectAtIndex:1];
    gameLevels = [hiScores objectAtIndex:2];

    [self displayText:@"Score"
                  AtX:230
                  AtY:400
             FontSize:20];

    [self displayText:@"Level"
                  AtX:350
                  AtY:400
             FontSize:20];

    [self displayText:@"Name"
                  AtX:450
                  AtY:400
             FontSize:20];

    for (loop = 0; loop < 10; loop++)
    {
        
        if(currentHighScoreRanking == loop)
        {
            [self displayText:[NSString stringWithFormat:@"%08d", [[gameScores objectAtIndex:loop] intValue]]
                          AtX:215
                          AtY:400 - (loop * 25) - 25 - 4
                     FontSize:24];

            [self displayText:[NSString stringWithFormat:@"%03d", [[gameLevels objectAtIndex:loop] intValue]]
                          AtX:345
                          AtY:400 - (loop * 25) - 25 - 4
                     FontSize:24];
            
            [self displayText:[[gameNames objectAtIndex:loop] stringByAppendingFormat:@"_"]
                          AtX:450
                          AtY:400 - (loop * 25) - 25 - 4
                     FontSize:24];            
        }
        else
        {
            [self displayText:[NSString stringWithFormat:@"%08d", [[gameScores objectAtIndex:loop] intValue]]
                          AtX:230
                          AtY:400 - (loop * 25) - 25
                     FontSize:16];

            [self displayText:[NSString stringWithFormat:@"%03d", [[gameLevels objectAtIndex:loop] intValue]]
                          AtX:350
                          AtY:400 - (loop * 25) - 25
                     FontSize:16];
            
            [self displayText:[gameNames objectAtIndex:loop]
                          AtX:450
                          AtY:400 - (loop * 25) - 25
                     FontSize:16];
        }
    }

    // Draw the buttons
    [[NSImage imageNamed:@"button_done"] compositeToPoint:NSMakePoint(262, 20) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_DONE ? 1.0 : 0.5];
}



- (void)drawShopDisplay
{
    ShopItem* item;
    
    // display the shop background image
    [[NSImage imageNamed:@"upgrade_shop"] compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];

    // draw inventory background box
    [[[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.5 alpha:1.0] colorWithAlphaComponent:0.5] set];

    // inventory background
    NSRectFillUsingOperation(NSMakeRect(300, 90, 200, 15), NSCompositeSourceOver);
    NSRectFillUsingOperation(NSMakeRect(300, 110, 200, 370), NSCompositeSourceOver);
    NSRectFillUsingOperation(NSMakeRect(300, 485, 200, 15), NSCompositeSourceOver);

    // shop item background
    //NSRectFillUsingOperation(NSMakeRect(510, 90, 280, 160), NSCompositeSourceOver);
    //NSRectFillUsingOperation(NSMakeRect(510, 255, 280, 245), NSCompositeSourceOver);
     
    // display current cash flow
    [self displayText:[NSString stringWithFormat:@"Credits: %06d",[game getMoney]]
                  AtX:305
                  AtY:90
             FontSize:12];

    [self displayText:@"Inventory"
                  AtX:305
                  AtY:485
             FontSize:12];
    
    
    // list the items already owned
    int loop;
    int entryY = 465;

    for(loop = 0; loop < [game getShopItemCount]; loop++)
    {
        item = [game getShopItemAt:loop];

        if([item isBought])
        {
            [self displayText:[item getTitle]
                          AtX:305
                          AtY:entryY
                     FontSize:12];
            
            entryY -= 15;
        }
    }

    // display Nova Bomb count
    if([game getNovaBombCount] > 0)
    {
        [self displayText:[NSString stringWithFormat:@"Nova Bombs: %d", [game getNovaBombCount]]
                      AtX:305
                      AtY:entryY
                 FontSize:12];
    }

    // display current item
    item = [game getShopItemAt:[game getCurrentShopItem]];

    // work out if the item is affordable
    double fade;
    if([game getMoney] < [item getCost])
    {
        fade = 0.25;
    }
    else
    {
        fade = 1.0;
    }

    [[item getImage] compositeToPoint:NSMakePoint(520, 260) operation:NSCompositeSourceOver fraction:fade];


    [self displayText:[item getTitle]
                  AtX:515
                  AtY:220
             FontSize:20];
    
    [self displayText:[item getDescription]:515:220:270:100];
    
    [self displayText:@"Cost"
                  AtX:515
                  AtY:110
             FontSize:12];

    [self displayText:[NSString stringWithFormat:@"%d",[item getCost]]
                  AtX:515
                  AtY:90
             FontSize:20];

    /*
    [self displayText:[NSString stringWithFormat:@"Owned: %@",[item isBought] ? @"Yes" : @"No"]
                  AtX:515
                  AtY:110
             FontSize:12];
     */

    // Draw the buttons
    [[NSImage imageNamed:@"button_next"] compositeToPoint:NSMakePoint(5, 350) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_NEXT ? 1.0 : 0.5];

    [[NSImage imageNamed:@"button_previous"] compositeToPoint:NSMakePoint(5, 280) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_PREVIOUS ? 1.0 : 0.5];

    [[NSImage imageNamed:@"button_buy"] compositeToPoint:NSMakePoint(5, 210) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_BUY ? 1.0 : 0.5];

    [[NSImage imageNamed:@"button_leave_shop"] compositeToPoint:NSMakePoint(5, 140) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_LEAVE_SHOP ? 1.0 : 0.5];
    
}

- (void)drawDroidShopDisplay
{
    // display the shop background image
    [[NSImage imageNamed:@"shop"] compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
    
    // droid selector background
    NSRectFillUsingOperation(NSMakeRect(10, 90, 280, 160), NSCompositeSourceOver);
    NSRectFillUsingOperation(NSMakeRect(10, 255, 280, 245), NSCompositeSourceOver);

    // display current cash flow
    [self displayText:@"Credits"
                  AtX:15
                  AtY:90
             FontSize:15];

    [self displayText:[NSString stringWithFormat:@"%06d",[game getMoney]]
                  AtX:15
                  AtY:70
             FontSize:20];

    // display the current robot selected
    double fade;
    if([game getMoney] < [[game getRobotAt:[game getCurrentRobotItem]] getCost])
    {
        fade = 0.25;
    }
    else
    {
        fade = 1.0;
    }

    [[[game getRobotAt:[game getCurrentRobotItem]] getImage] compositeToPoint:NSMakePoint(15, 250) operation:NSCompositeSourceOver fraction:fade];

    [self displayText:[[game getRobotAt:[game getCurrentRobotItem]] getTitle]
                  AtX:15
                  AtY:220
             FontSize:20];
    
    [self displayText:[[game getRobotAt:[game getCurrentRobotItem]] getDescription]:15:220:270:100];
    
    [self displayText:@"Speed"
                  AtX:15
                  AtY:145
             FontSize:12];

    [self displayText:[NSString stringWithFormat:@"%d", [[game getRobotAt:[game getCurrentRobotItem]] getSpeed]]
                  AtX:15
                  AtY:125
             FontSize:20];
    
    [self displayText:@"Ammo Capacity"
                  AtX:60
                  AtY:145
             FontSize:12];

    [self displayText:[NSString stringWithFormat:@"%d", [[game getRobotAt:[game getCurrentRobotItem]] getAmmoCapacity]]
                  AtX:60
                  AtY:125
             FontSize:20];

    [self displayText:@"Fruit Capacity"
                  AtX:165
                  AtY:145
             FontSize:12];

    [self displayText:[NSString stringWithFormat:@"%d", [[game getRobotAt:[game getCurrentRobotItem]] getCarryCapacity]]
                  AtX:165
                  AtY:125
             FontSize:20];
    
    [self displayText:@"Cost"
                  AtX:270
                  AtY:145
             FontSize:12];

    [self displayText:[NSString stringWithFormat:@"%d",[[game getRobotAt:[game getCurrentRobotItem]] getCost]]
                  AtX:270
                  AtY:125
             FontSize:20];
    

    // Draw the buttons
    [[NSImage imageNamed:@"button_next"] compositeToPoint:NSMakePoint(400, 350) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_NEXT ? 1.0 : 0.5];

    [[NSImage imageNamed:@"button_previous"] compositeToPoint:NSMakePoint(400, 280) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_PREVIOUS ? 1.0 : 0.5];

    [[NSImage imageNamed:@"button_buy"] compositeToPoint:NSMakePoint(400, 210) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_BUY ? 1.0 : 0.5];

    [[NSImage imageNamed:@"button_leave_shop"] compositeToPoint:NSMakePoint(400, 140) operation:NSCompositeSourceOver fraction: currentMenuItem == BUTTON_LEAVE_SHOP ? 1.0 : 0.5];

}



- (void)displayText:(NSString*)text AtX:(int)x AtY:(int)y FontSize:(int)fontSize
{
    NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithCapacity:0];
    [attr setObject:[NSColor whiteColor]
             forKey:NSForegroundColorAttributeName];

    [attr setObject:[NSFont fontWithName:@"Geneva" size:fontSize]
              forKey:NSFontAttributeName];

    
    NSPoint ptText = NSMakePoint(x, y);

    [text drawAtPoint:ptText withAttributes:attr];

    
}

- (void)displayText:(NSString*)text:(int)x:(int)y:(int)width:(int)height
{
    NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithCapacity:0];
    [attr setObject:[NSColor whiteColor] forKey:NSForegroundColorAttributeName];

    NSRect rcText = NSMakeRect(x, y - height, width, height);

    [text drawInRect:rcText withAttributes:attr];
}

- (NSString*)displayKey:(int)key
{
    NSString* sKey;

    switch(key)
    {
        case NSUpArrowFunctionKey:
            sKey = @"Up Arrow";
            break;

        case NSDownArrowFunctionKey:
            sKey = @"Down Arrow";
            break;

        case NSLeftArrowFunctionKey:
            sKey = @"Left Arrow";
            break;

        case NSRightArrowFunctionKey:
            sKey = @"Right Arrow";
            break;

        case (int)' ':
            sKey = @"Space";
            break;

        default:
            sKey = [NSString stringWithFormat:@"%c", key];
            break;
    }

    return sKey;
}

- (void)mouseDown:(NSEvent*)event
{
    NSPoint eventLocation = [event locationInWindow];
    NSPoint centre = [self convertPoint:eventLocation fromView:nil];

    // remember this mouse position for use when setting the rectangles
    lastMousePoint = centre;

    switch([game getGameStatus])
    {
        case GAME_PREFS:
            switch(currentMenuItem)
            {
                case BUTTON_DONE:
                    [game setGameStatus:GAME_TITLE];
                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    break;

                case BUTTON_SOUND:
                    if([game isSoundOn])
                    {
                        [game setSoundOn:NO];
                    }
                    else
                    {
                        [game setSoundOn:YES];
                    }                    
                    break;

                case BUTTON_MUSIC:
                    if([game isMusicOn])
                    {
                        [game setMusicOn:NO];
                        [controller musicOnOff:NO];
                        
                        [self setNeedsDisplay:YES];
                    }
                    else
                    {
                        [game setMusicOn:YES];
                        [controller musicOnOff:YES];

                        [self setNeedsDisplay:YES];
                    }
                    break;
                    
            }
            break;

        case GAME_OVER:
            switch(currentMenuItem)
            {
                case BUTTON_CONTINUE:
                    currentHighScoreRanking = [controller checkHighScores];
                    
                    if(currentHighScoreRanking != -1)
                    {
                        NSMutableArray	*gameScores, *gameNames, *gameLevels;
                        NSArray* hiScores;

                        hiScores = [controller getHiScores];

                        // reset arrays with gameLevel
                        gameNames = [hiScores objectAtIndex:0];
                        gameScores = [hiScores objectAtIndex:1];
                        gameLevels = [hiScores objectAtIndex:2];

                        [gameScores insertObject:[NSNumber numberWithInt:[game getScore]]
                                         atIndex:currentHighScoreRanking];
                        
                        [gameScores removeObjectAtIndex:10];
                        
                        [gameNames insertObject:@""
                                        atIndex:currentHighScoreRanking];
                        
                        [gameNames removeObjectAtIndex:10];
                        
                        [gameLevels insertObject:[NSNumber numberWithInt:[game getLevel]]
                                         atIndex:currentHighScoreRanking];
                        
                        [gameLevels removeObjectAtIndex:10];
                        
                        [[NSUserDefaults standardUserDefaults] setObject:hiScores
                                                                  forKey:@"hiScores"];
                        
                        [game setGameStatus:GAME_NEW_HIGH_SCORE];
                    }
                    else
                    {
                        [game setGameStatus:GAME_TITLE];
                    }
                    
                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    break;
            }
            break;

        case GAME_HIGH_SCORES:
            switch(currentMenuItem)
            {
                case BUTTON_CONTINUE:
                    [game setGameStatus:GAME_TITLE];
                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    break;
            }
            break;

        case GAME_NEW_HIGH_SCORE:
            switch(currentMenuItem)
            {
                case BUTTON_DONE:

                   
                    
                    [game setGameStatus:GAME_TITLE];
                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    break;
            }
            break;

        case GAME_LEVEL_INTRO:
            switch(currentMenuItem)
            {
                case BUTTON_CONTINUE:
                    [game setGameStatus:GAME_PLAYING];
                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    break;
            }
            break;

        case GAME_LEVEL_OVER:
            switch(currentMenuItem)
            {
                case BUTTON_CONTINUE:
                    [game changeLevel:1];
                    [game setGameStatus:GAME_MAIN_MENU];
                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    break;
            }
            break;

        case GAME_LIFE_OVER:
            switch(currentMenuItem)
            {
                case BUTTON_CONTINUE:
                    // Does the player have an insurance policy for the droid?
                    if([game isInsuranceOwned])
                    {
                        // remove the insurance policy
                        [game removeShopItem:ITEM_INSURANCE];
                        
                        [game setGameStatus:GAME_MAIN_MENU];
                        [self setTrackingAreas];
                    }
                    else
                    {

                        // Does the player have enough money to buy a new droid
                        if([game getMoney] >= 10000)
                        {
                            // take the destroyed droid away from the player
                            [game setCurrentRobot:-1];

                            [game setGameStatus:GAME_MAIN_MENU];
                            [self setTrackingAreas];
                        }
                        else
                        {
                            [timer invalidate];

                            if([game getScore] > [game getHighScore])
                            {
                                [game setHighScore:[game getScore]];
                            }

                            [game setGameStatus:GAME_OVER];
                            [self setTrackingAreas];
                        }
                    }
                        
                    [self setNeedsDisplay:YES];
                    break;
            }
            break;
            
        case GAME_TITLE:
            switch(currentMenuItem)
            {
                case BUTTON_UDEVGAME:
                    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://www.idevgames.com/content/contest_vote.php?id=10"]];
                    break;

                case BUTTON_HIGH_SCORES:
                    [game setGameStatus:GAME_HIGH_SCORES];

                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    break;
                    
                case BUTTON_OPTIONS:
                    [game setGameStatus:GAME_PREFS];

                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    break;

                case BUTTON_PLAY:
                    [game newGame];
                    [game setGameStatus:GAME_MAIN_MENU];

                    timer = [NSTimer	scheduledTimerWithTimeInterval:0
                                                             target:self
                                                           selector:@selector(runTimer)
                                                           userInfo:self
                                                            repeats:YES];

                    [self setTrackingAreas];

                    [self setNeedsDisplay:YES];
                    
                    break;
            }
            break;
            
        case GAME_DROID_SHOP:
            switch(currentMenuItem)
            {
                case BUTTON_NEXT:
                    [game changeCurrentRobotItem:1];
                    [self setNeedsDisplay:YES];

                    break;

                case BUTTON_PREVIOUS:
                    [game changeCurrentRobotItem:-1];
                    [self setNeedsDisplay:YES];
                    break;

                case BUTTON_BUY:
                    if([game getMoney] -  [[game getRobotAt:[game getCurrentRobotItem]] getCost] >= 0)
                    {
                        [game changeMoney:- [[game getRobotAt:[game getCurrentRobotItem]] getCost]];
                        [game setCurrentRobot:[game getCurrentRobotItem]];

                        [self setNeedsDisplay:YES];
                    }
                    
                    break;

                case BUTTON_LEAVE_SHOP:
                    [game setGameStatus:GAME_MAIN_MENU];
                    [self setTrackingAreas];
                    [self setNeedsDisplay:YES];

                    break;
            }
            break;

        case GAME_SHOP:
            switch(currentMenuItem)
            {
                case BUTTON_NEXT:
                    [game changeCurrentShopItem:1];
                    [self setNeedsDisplay:YES];

                    break;

                case BUTTON_PREVIOUS:
                    [game changeCurrentShopItem:-1];
                    [self setNeedsDisplay:YES];
                    break;

                case BUTTON_BUY:
                {
                    ShopItem* item = [game getShopItemAt:[game getCurrentShopItem]];

                    if([game getMoney] - [item getCost] >= 0
                       && ![item isBought])
                    {
                        [item setBought:YES];

                        [game changeMoney:-[item getCost]];

                        if([game getCurrentShopItem] == ITEM_NOVA_BOMB)
                        {
                            // don't set owned flag for Nova Bombs
                            [item setBought:NO];

                            [game changeNovaBombCount:1];
                        }
                        [self setNeedsDisplay:YES];
                    }
                }

                    break;
                    
                case BUTTON_LEAVE_SHOP:
                    [game setGameStatus:GAME_MAIN_MENU];
                    [self setTrackingAreas];
                    [self setNeedsDisplay:YES];

                    break;
            }
            break;
            
            
        case GAME_MAIN_MENU:
            switch(currentMenuItem)
            {
                case MENU_UPGRADES: // Upgrades 4 U?
                    if([game getCurrentRobot] != -1)
                    {

                        [game setGameStatus:GAME_SHOP];

                        [self setTrackingAreas];
                        [self setNeedsDisplay:YES];
                    }
                    break;

                case MENU_DOMES:  // Growing Domes
                    if([game getCurrentRobot] != -1)
                    {
                        [game setBonusLevel:NO];
                        [game clearAllObjects];
                        [game initLevel];

                        [game setGameStatus:GAME_LEVEL_INTRO];

                        [self setTrackingAreas];
                        [self setNeedsDisplay:YES];
                    }
                    break;

                case MENU_DROID_SHOP:  // Droid Emporium

                    [game setGameStatus:GAME_DROID_SHOP];

                    [self setTrackingAreas];
                    [self setNeedsDisplay:YES];
                    break;
            }
                      
            break;
    }

}


- (void)mouseEntered:(NSEvent*)event
{
    currentMenuItem = [event userData];
   [self setNeedsDisplay:YES];
}

- (void)mouseExited:(NSEvent*)event
{
    if(currentMenuItem == [event userData])
    {
        currentMenuItem = MENU_NONE;
    }
    
    [self setNeedsDisplay:YES];

}


- (void)mouseMoved:(NSEvent*)event
{
    NSPoint eventLocation = [event locationInWindow];
    NSPoint centre = [self convertPoint:eventLocation fromView:nil];

    switch([game getGameStatus])
    {
        case GAME_MAIN_MENU:

            // Upgrades 4 U?
            if([game getCurrentRobot] != -1)
            {
                if(centre.x >= 480 && centre.x < 795 && centre.y >= 350 && centre.y < 586)
                {
                    currentMenuItem = MENU_UPGRADES;
                    
                    [self setNeedsDisplay:YES];
                }
            }

            // Droid Emporium
            else if(centre.x >= 5 && centre.x < 320 && centre.y >= 350 && centre.y < 586)
            {
                currentMenuItem = MENU_DROID_SHOP;
                [self setNeedsDisplay:YES];
            }


            else
            {
                currentMenuItem = -1;
                [self setNeedsDisplay:YES];
            }

            break;
    }

}

- (void)setTrackingAreas
{
   
    int loop;
    
    // remove existing areas
    for(loop = 0; loop < BUTTONS_LAST; loop++)
    {
        if(buttons[loop] != -1)
        {
            [self removeTrackingRect:buttons[loop]];
        }
    }

    currentMenuItem = MENU_NONE;


    switch([game getGameStatus])
    {
        case GAME_MAIN_MENU:

            [self createTrackingArea:NSMakeRect(5, 320, 315, 236)
                             AndItem:MENU_DROID_SHOP];

            [self createTrackingArea:NSMakeRect(480, 350, 315, 236)
                             AndItem:MENU_UPGRADES];

            [self createTrackingArea:NSMakeRect(242, 100, 315, 236)
                             AndItem:MENU_DOMES];
            break;

        case GAME_DROID_SHOP:

            [self createTrackingArea:NSMakeRect(400, 350, 276, 70)
                             AndItem:BUTTON_NEXT];

            [self createTrackingArea:NSMakeRect(400, 280, 276, 70)
                             AndItem:BUTTON_PREVIOUS];

            [self createTrackingArea:NSMakeRect(400, 210, 276, 70)
                             AndItem:BUTTON_BUY];

            [self createTrackingArea:NSMakeRect(400, 140, 276, 70)
                             AndItem:BUTTON_LEAVE_SHOP];
            break;

        case GAME_SHOP:
            
            [self createTrackingArea:NSMakeRect(5, 350, 276, 70)
                             AndItem:BUTTON_NEXT];

            [self createTrackingArea:NSMakeRect(5, 280, 276, 70)
                             AndItem:BUTTON_PREVIOUS];

            [self createTrackingArea:NSMakeRect(5, 210, 276, 70)
                             AndItem:BUTTON_BUY];

            [self createTrackingArea:NSMakeRect(5, 140, 276, 70)
                             AndItem:BUTTON_LEAVE_SHOP];
            break;

        case GAME_TITLE:
            [self createTrackingArea:NSMakeRect(500, 350, 276, 70)
                             AndItem:BUTTON_OPTIONS];

            [self createTrackingArea:NSMakeRect(500, 280, 276, 70)
                             AndItem:BUTTON_HIGH_SCORES];

            [self createTrackingArea:NSMakeRect(500, 210, 276, 70)
                             AndItem:BUTTON_PLAY];


            [self createTrackingArea:NSMakeRect(700, 480, 100, 117)
                             AndItem:BUTTON_UDEVGAME];
            break;

        case GAME_PREFS:
            [self createTrackingArea:NSMakeRect(262, 10, 276, 70)
                             AndItem:BUTTON_DONE];

            [self createTrackingArea:NSMakeRect(450, 350, 276, 70)
                             AndItem:BUTTON_MUSIC];

            [self createTrackingArea:NSMakeRect(450, 300, 276, 70)
                             AndItem:BUTTON_SOUND];
            
            break;

        case GAME_LEVEL_OVER:

            [self createTrackingArea:NSMakeRect(262, 20, 276, 70)
                             AndItem:BUTTON_CONTINUE];

            break;

        case GAME_LIFE_OVER:
            
            [self createTrackingArea:NSMakeRect(262, 20, 276, 70)
                             AndItem:BUTTON_CONTINUE];

            break;

        case GAME_OVER:
            
            [self createTrackingArea:NSMakeRect(262, 20, 276, 70)
                             AndItem:BUTTON_CONTINUE];

            break;

        case GAME_LEVEL_INTRO:

            [self createTrackingArea:NSMakeRect(262, 20, 276, 70)
                             AndItem:BUTTON_CONTINUE];

            break;

        case GAME_HIGH_SCORES:

            [self createTrackingArea:NSMakeRect(262, 20, 276, 70)
                             AndItem:BUTTON_CONTINUE];

            break;

        case GAME_NEW_HIGH_SCORE:

            [self createTrackingArea:NSMakeRect(262, 20, 276, 70)
                             AndItem:BUTTON_DONE];

            break;
            
            
            
    }


    
}


- (void)createTrackingArea:(NSRect)rect AndItem:(int)item
{
    if(NSPointInRect(lastMousePoint, rect))
    {
        buttons[item] = [self addTrackingRect:rect owner:self userData:(void*)item assumeInside:YES];

        currentMenuItem = item;
    }
    else
    {
        buttons[item] = [self addTrackingRect:rect owner:self userData:(void*)item assumeInside:NO];
    }
}
@end
