//
//  Plant.m
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import "Plant.h"
#import "Sprite.h"

@implementation Plant

- (id) init
{
    self = [super init];

    if (spriteArray)
    {
        [spriteArray release];
    }

    spriteArray = [[NSMutableArray arrayWithCapacity:0] retain];

    [self setHealth:5];
    [self setTargetHeight:30];

    return self;
}

- (void) dealloc
{
    if (spriteArray)
    {
        [spriteArray release];
    }
    
    [super dealloc];
}

- (void)setPosition:(int)x:(int)y
{
    position.x = x;
    position.y = y;
}

- (int)getPositionX
{
    return position.x;
}

- (int)getPositionY
{
    return position.y;
}

- (void)setHeight:(int)newHeight
{
    height = newHeight;
}

- (void)setTargetHeight:(int)newHeight
{
    targetHeight = newHeight;
}

- (int)getTargetHeight
{
    return targetHeight;
}

- (bool)isFullyGrown
{
    return [self getTargetHeight] == [self getHeight];
}

- (void)growPlant:(int)change
{
    if(height + change < [self getTargetHeight] + 1)
    {
        height += change;

        [self addSprite];
    }
}

- (void)killPlant:(int)change
{
    [self changeHealth:-1];

    if([self getHealth] == 0)
    {
        if(height - change >= 0)
        {
            height -= change;
            [self removeSprite];
        }

        [self setHealth:5];
    }
}

- (int)getHeight
{
    return height;
}

- (void)setHealth:(int)newHealth
{
    health = newHealth;
}

- (void)changeHealth:(int)change
{
    health += change;
}

- (int)getHealth
{
    return health;
}

- (void)addSprite
{
    CRSprite* newSprite;

    newSprite = [[[CRSprite alloc] init] retain];

    [newSprite setSize:30:30];
    [newSprite setMovement:0:0];
    [newSprite setPosition:[self getPositionX]:(10 * height) + [self getPositionY]];
    [newSprite loadImageArray:@"plant":1];
    [newSprite setFrames:1];

    [spriteArray addObject:newSprite];
}

- (void)removeSprite
{
    int count = [spriteArray count];
    [spriteArray removeObjectAtIndex:count - 1];
}

- (void)drawPlant
{

    CRSprite* sprite;
    int loop;

    for(loop = 0; loop < [spriteArray count]; loop++)
    {
        sprite = [spriteArray objectAtIndex:loop];
        [sprite drawSprite:1:0:0];
        
    }
}

- (bool)doesAlienEatPlant:(int)x:(int)y:(int)w:(int)h
{
    CRSprite* sprite;
    int loop;
    NSPoint ptSprite;
    NSRect rcSprite;
    NSRect rcCheck = NSMakeRect(x, y, w - 1, h - 1);

    for(loop = 0; loop < [spriteArray count]; loop++)
    {
        sprite = [spriteArray objectAtIndex:loop];

        ptSprite = [sprite getPosition];

        rcSprite = NSMakeRect(ptSprite.x, ptSprite.y, 30 - 1, 30 - 1);

        if(NSIntersectsRect(rcSprite, rcCheck))
        {
            return YES;
        }
        
    }

    return NO;
}

@end
