//
//  Sprite.h
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import <Foundation/Foundation.h>

@class Game;
@class Tile;

@interface CRSprite : NSObject {

    NSPoint	position;	// on screen position
    int		rotation;	// rotation of tile 0 - 12, 1 - 3, 2 - 6, 3 - 9
    NSImage*	image;		// sprite's image
    int		width;		// sprite width
    int		height;		// sprite height
    int		frames;		// number of frames
    int		currentFrame;	// current frame
    int		deltaX;		// x movement
    int		deltaY;		// y movement

    bool	bounce;		// bounce off walls?
    bool	gravity;	// gravity on?

    Game*	theGame;	// pointer to game object

    NSMutableArray		*spriteImageArray;	// array of frame images for sprite
    NSMutableArray		*pathArray;		// flight path array
}

- (id) init;
- (void) dealloc;

- (void)setGame:(Game*)aGame;

- (void)setBounceOn:(bool)state;
- (bool)isBounceOn;

- (void)setGravityOn:(bool)state;
- (bool)isGravityOn;

- (void) setRotation: (int)newRotation;

- (void) rotate: (int)amount;

- (void) setPosition:(int)x: (int)y;
- (NSPoint) getPosition;

- (void) setSize:(int)newWidth: (int)newHeight;
- (int) getWidth;
- (int) getHeight;

- (void) moveSprite;
- (void) setMovement:(int)x:(int)y;
- (int) getMovementX;
- (int) getMovementY;

- (void) setFrames:(int)numberOfFrames;
- (void) setCurrentFrame:(int)newFrame;
- (void) changeFrame:(int)change;


- (void) setImage:(NSImage *)newImage;
- (NSImage*) getImage;


- (void) loadImageArray:(NSString*)filename:(int)numberOfFrames;
- (void) createPathArray;

- (void) drawSprite:(float)scale:(int)offsetX:(int)offsetY;

@end
