//
//  Sprite.m
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import "Sprite.h"
#import "Game.h"
#import "Tile.h"

@implementation CRSprite


- (id) init
{
    self = [super init];

    rotation = 0;

    bounce = NO;
    gravity = NO;
    return self;
}

- (void) dealloc
{
    if (spriteImageArray)
    {
        [spriteImageArray release];
    }

    if(pathArray)
    {
        [pathArray release];
    }
    
    [super dealloc];
}

- (void)setGame:(Game*)aGame
{
    theGame = aGame;
}

- (void)setBounceOn:(bool)state
{
    bounce = state;
}

- (bool)isBounceOn
{
    return bounce;
}

- (void)setGravityOn:(bool)state
{
    gravity = state;
}

- (bool)isGravityOn
{
    return gravity;
}

- (void) setRotation: (int)newRotation
{
    rotation = newRotation;
}


- (void) rotate: (int)amount
{
    rotation += amount;
}

- (void) setPosition:(int)x: (int)y
{
    position.x = x;
    position.y = y;
}

- (NSPoint) getPosition
{
    return position;
}

- (void) setSize:(int)newWidth: (int)newHeight
{
    width = newWidth;
    height = newHeight;
}

- (int) getWidth
{
    return width;
}

- (int) getHeight
{
    return height;
}

- (void) moveSprite
{
    int loop;
    Tile* tile;

    BOOL blockX = NO;
    BOOL blockY = NO;
    

    if(gravity && deltaY > -5)
    {
        deltaY -= 5;
    }
    

    // check for player hitting tile
    for(loop = 0; loop < [theGame getTileCount]; loop++)
    {
        tile = [theGame getTileAt:loop];

        if([tile getType] == 0)
        {
            if([tile isWithinRect:position.x + deltaX:position.y:60:60])
            {
                if(bounce)
                {
                    deltaX *= -1;
                }
                else
                {
                    deltaX = 0;
                }

                blockX = YES;
            }
            if([tile isWithinRect:position.x:position.y + deltaY:60:60])
            {
                if(bounce)
                {
                    deltaY *= -1;
                }
                else
                {
                    deltaY = 0;
                }

                blockY = YES;
            }
        }
    }

    if(!blockX)
    {
        position.x += deltaX;
    }

    if(!blockY)
    {
        position.y += deltaY;
    }

    /*
        

    

    if(position.x < 0)
    {
        if(bounce)
        {
            deltaX *= -1;
        }
        else
        {
            position.x = 0;
        }
    }

    if(position.x > 799 - width)
    {
        if(bounce)
        {
            deltaX *= -1;
        }
        else
        {
            position.x = 799 - width;
        }
    }

    if(position.y < 0)
    {
        if(bounce)
        {
            deltaY *= -1;
        }
        else
        {
            position.y = 0;
        }
    }

    if(position.y > 599 - height)
    {
        if(bounce)
        {
            deltaY *= -1;
        }
        else
        {
            position.y = 599 - height;
        }
    }
     */
}

- (void) setMovement:(int)x:(int)y
{
    deltaX = x;
    deltaY = y;
}

- (int) getMovementX
{
    return deltaX;
}

- (int) getMovementY
{
    return deltaY;
}

- (void) setFrames:(int)numberOfFrames
{
    frames = numberOfFrames;
}

- (void) setCurrentFrame:(int)newFrame
{
    currentFrame = newFrame;
}

- (void) changeFrame:(int)change
{
    currentFrame = (currentFrame + change) % frames;
}


- (void) setImage:(NSImage *)newImage
{
    image = newImage;
}

- (NSImage*) getImage
{
    return image;
}

- (void) loadImageArray:(NSString*)filename:(int)numberOfFrames
{
    int loop;
    NSString* sFilename;
    NSImage* imageNew;
    
    
    if (spriteImageArray)
    {
        [spriteImageArray release];
    }
    
    spriteImageArray = [[NSMutableArray arrayWithCapacity:0] retain];

    for(loop = 1; loop < numberOfFrames + 1; loop++)
    {
        sFilename = [NSString stringWithFormat:@"%@%d",
            filename,
            loop];

        imageNew = [[NSImage imageNamed:sFilename] retain];

        // Do not check it with the file periodically - it won't change
        [imageNew setDataRetained:YES];

        // Put it in memory
        [imageNew lockFocus];
        [imageNew unlockFocus];
        
        [spriteImageArray addObject:imageNew];
    }


    for(loop = numberOfFrames - 1; loop > 1; loop--)
    {
        sFilename = [NSString stringWithFormat:@"%@%d",
            filename,
            loop];

        imageNew = [[NSImage imageNamed:sFilename] retain];

        // Do not check it with the file periodically - it won't change
        [imageNew setDataRetained:YES];

        // Put it in memory
        [imageNew lockFocus];
        [imageNew unlockFocus];

        [spriteImageArray addObject:imageNew];
    }
        
}

- (void) createPathArray
{
    if(pathArray)
    {
        [pathArray release];
    }

    pathArray = [[NSMutableArray arrayWithCapacity:0] retain];


    //NSPoint pt = NSMakePoint(5, 5);
    //[pathArray addObject:&pt];
    
    //[pathArray addObject:[NSPoint pt]];
}

- (void) drawSprite:(float)scale:(int)offsetX:(int)offsetY
{
    //NSImage* imageThis;
    //imageThis = [self getImage];

    //imageThis = [spriteImageArray objectAtIndex:currentFrame];

    /*
    [imageThis drawInRect:NSMakeRect(position.x, position.y, width, height)
                 fromRect:NSMakeRect(0, 0, width, height)
                operation:NSCompositeSourceOver
                 fraction:1.0];

     */

    // draw bounding box for collison debug
    /*
    NSRect rcBox = NSMakeRect(position.x, position.y, width, height);
    NSColor *fillColour;

    fillColour = [[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.5 alpha:1.0] colorWithAlphaComponent:0.5];

    [fillColour set];

    NSRectFillUsingOperation(rcBox, NSCompositeSourceOver);
     */
    
    [[spriteImageArray objectAtIndex:currentFrame] compositeToPoint:NSMakePoint(position.x, position.y) operation:NSCompositeSourceOver];
    
    /*
    NSSize size = NSMakeSize(width / scale, height / scale);

    NSAffineTransform *transform = [NSAffineTransform transform];

    [imageThis setScalesWhenResized:YES];

    [imageThis setSize:size];


    
    [transform scaleXBy:scale yBy:scale];
    [transform translateXBy:position.x + offsetX yBy:position.y + offsetY];
    [transform rotateByDegrees:rotation];

    [transform concat];
     */

    /*
    [imageThis drawInRect:NSMakeRect(-(width / 2), -(height / 2), width, height)
                 fromRect:NSMakeRect(0, 0, width, height)
                operation:NSCompositeSourceOver
                 fraction:1.0];

     */
     /*
    [transform invert];
    [transform concat];
 */
}


@end
