//
//  Tile.h
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import <Foundation/Foundation.h>

@class CRSprite;

#define TILE_WIDTH	60
#define TILE_HEIGHT	60


@interface Tile : NSObject {

    CRSprite*	sprite;		// sprite for displaying tile
    NSPoint	position;	// on screen position
    int		type;		// type of tile
    int		rotation;	// rotation of tile 0 - 12, 1 - 3, 2 - 6, 3 - 9
    NSImage*	image;		// tile's image
    int		life;		// number of frames before destroyed -1 live forever
    int		fireX;
    int		fireY;
    int		animateDelay;	// current animation delay
    int		animateCounter;	// delay counter
}

- (id) init;
- (void) dealloc;

- (void) setType: (int)newType;
- (int) getType;

- (void)setAnimationDelay:(int)newDelay;
- (void)resetAnimationCounter;

- (void) setFireDirection:(int)newFireX:(int)newFireY;
- (int) getFireX;
- (int) getFireY;

- (void) setRotation: (int)newRotation;

- (void) rotate: (int)amount;

- (void) setPosition:(int)x: (int)y;
- (NSPoint) getPosition;

- (void) setImage:(NSImage *)newImage;
- (NSImage*) getImage;

- (CRSprite*) getSprite;

- (void) drawTile:(int)scale:(int)offsetX:(int)offsetY;

- (void)animate;

- (void)setLife:(int)newLife;
- (void)decreaseLife;
- (int)getLife;

- (bool)isWithinRect:(int)x:(int)y:(int)w:(int)h;

@end
