//
//  Tile.m
//  Garden Pests
//
//  Copyright (c) 2003 Andrew Sage. All rights reserved.
//
//  This code is released under the uDevGames license.
//

#import "Tile.h"
#import "Sprite.h"


@implementation Tile

- (id) init
{
    self = [super init];

    sprite = [[[CRSprite alloc] init] retain];

    [sprite setSize:TILE_WIDTH:TILE_HEIGHT];
    [sprite loadImageArray:@"frame":4];
    [sprite setFrames:6];
    [sprite setCurrentFrame:0];
    //[sprite setMovement:(rand()%10) - 5:(rand()%10) - 5];
    //[sprite setMovement:((rand()%10) - 5) * 1:((rand()%10) - 5) * 1];
    [sprite setMovement:0:0];


    [self setFireDirection:5:0];
    life = -1;

    fireY = 5;

    [self setAnimationDelay:5];
        
    return self;
}

- (void) dealloc
{
    [sprite release];
    [super dealloc];
}

- (void) setFireDirection:(int)newFireX:(int)newFireY
{
    fireX = newFireX;
    fireY = newFireY;
}
    
- (int) getFireX
{
    return fireX;
}

- (int) getFireY
{
    return fireY;
}

- (void) setType: (int)newType
{
    type = newType;
}

- (int) getType
{
    return type;
}


- (void) setRotation: (int)newRotation
{
    [sprite setRotation:newRotation];
}


- (void) rotate: (int)amount
{
    [sprite rotate:amount];
}

- (void) setPosition:(int)x: (int)y
{
    [sprite setPosition:x:y];
}

- (NSPoint) getPosition
{
    return [sprite getPosition];
}

- (void) setImage:(NSImage *)newImage
{
    //[sprite setImage:newImage];

    image = newImage;
}

- (NSImage*) getImage
{
    return image;
}

- (CRSprite*) getSprite
{
    return sprite;
}


- (void) drawTile:(int)scale:(int)offsetX:(int)offsetY
{
    if([self getLife] != 0)
    {
        [sprite drawSprite:scale:offsetX:offsetY];
    }
}

- (void)animate
{
    animateCounter--;

    if(animateCounter < 0)
    {
        [self resetAnimationCounter];
        [sprite changeFrame:1];
    }

    [sprite moveSprite];

    [self decreaseLife];
}

- (void)setAnimationDelay:(int)newDelay
{
    
    animateDelay = newDelay;

    [self resetAnimationCounter];

}

- (void)resetAnimationCounter
{
    animateCounter = animateDelay;
}

- (void)setLife:(int)newLife
{
    life = newLife;
}

- (void)decreaseLife
{
    if(life > 0)
    {
        life--;
    }
}

- (int)getLife
{
    return life;
}

- (bool)isWithinRect:(int)x:(int)y:(int)w:(int)h
{
    NSPoint ptSprite = [sprite getPosition];

    NSRect rcSprite = NSMakeRect(ptSprite.x, ptSprite.y, [sprite getWidth] - 1, [sprite getHeight] - 1);
    NSRect rcCheck = NSMakeRect(x, y, w - 1, h - 1);

    return NSIntersectsRect(rcSprite, rcCheck);
    /*

    // check to see if sprite is in target rectangle
    if(ptSprite.x > x && ptSprite.x < x + w
       && ptSprite.y > y && ptSprite.y < y + h)
    {
        return YES;
    }

    if(ptSprite.x + 60 > x && ptSprite.x + 60 < x + w
       && ptSprite.y + 60 > y && ptSprite.y + 60 < y + h)
    {
        return YES;
    }

    // check to see if targe rectangle is in sprite
    if(x > ptSprite.x && x < ptSprite.x + 60
       && y > ptSprite.y && y < ptSprite.y + 60)
    {
        return YES;
    }

    if(x + w > ptSprite.x && x + w < ptSprite.x + 60
       && y + h > ptSprite.y && y + h < ptSprite.y + 60)
    {
        return YES;
    }
    
    return NO;
     */
}

@end
